#! /usr/bin/env python
import aipy as a, ephem as e, numpy as n, sys, optparse, os

o = optparse.OptionParser()
o.set_usage('modmap.py [options]')
o.set_description(__doc__)
o.add_option('-s', '--srcs', dest='srcs', default='',
    help='The sources to add to the map.')
o.add_option('-n', '--nside', dest='nside', type='int', default=32,
    help='The NSIDE parameter for the Healpix Map.  Default is 32.')
o.add_option('-m', '--map', dest='map', default='out.fits',
    help='The location to save the map.')
o.add_option('-i', '--in_map', dest='in_map', 
    help='Existing map to add sources to.  Treated as temperature, i.e. if resolution is degraded, pixels are averaged.  Use mscale to re-scale if you want Jy per pixel.')
o.add_option('-f', '--freq', dest='freq', type='float', default=.150,
    help='Frequency to use for source strengths.  Default .150 GHz.')
o.add_option('--nindices', dest='nindices', type='int',default=1,
    help='Number of terms in the spectral index polynomial.')
o.add_option('--mscale', dest='mscale', type='float',default=1,
    help='Scaling to apply to data in map.  Useful for changing T into Jy per pixel, and reverse.')
o.add_option('--sscale', dest='sscale', type='float',default=1,
    help='Scaling to apply to sources.  Useful for changing Jy to T.  It may be useful to know that the size of a map pixel is 4*pi / (12*nside**2).')
o.add_option('--iepoch', dest='iepoch', default=e.J2000,
    help='The epoch of coordinates in the input map.  Default J2000.')
o.add_option('--oepoch', dest='oepoch', default=e.J2000, 
    help='The epoch of coordinates in the output map.  Default J2000.')
o.add_option('--isys', dest='isys', default='eq',
    help='Coordinates (eq,ec,ga) of input map.  Default is eq')
o.add_option('--osys', dest='osys', default='eq',
    help='Coordinates (eq,ec,ga) of output map.  Default is eq')

opts,args = o.parse_args(sys.argv[1:])

print 'Starting a new map with NSIDE=%d' % opts.nside
h = a.map.Map(nside=opts.nside, nindices=opts.nindices)
if not opts.in_map is None:
    print 'Importing data from %s' % opts.in_map
    h.from_map(a.map.Map(fromfits=opts.in_map))
    h.map.map *= opts.mscale
    px = n.arange(h.npix())
    m = a.coord.convert_m(opts.osys, opts.isys,
        iepoch=opts.oepoch, oepoch=opts.iepoch)
    x,y,z = n.dot(m,h.px2crd(px, ncrd=3))
    h.set_interpol(True)
    h.map[px] = h.map[x,y,z]
    h.wgt[px] = h.wgt[x,y,z]
    for i in h.ind: i[px] = i[x,y,z]
    h.set_interpol(False)
h.reset_wgt()

m = a.coord.convert_m('eq', opts.osys, oepoch=opts.oepoch)
for src in opts.srcs.split(','):
    if src == '': continue
    ra,dec,flux,mfreq,index,size = a.src.src_data[src]
    eq = e.Equatorial(ra, dec, epoch=e.J2000)
    eq = a.coord.radec2eq(eq.get())
    x,y,z = n.dot(m, eq)
    strength = flux * (opts.freq / mfreq)**index * opts.sscale
    print 'Adding', src, 'with strength %f Jy' % strength,
    print 'and index %f' % index
    if opts.nindices == 0:
        curflux = h[x,y,z]
        h.put((x,y,z),1,strength+curflux)
    else:
        curflux, i = h[x,y,z]
        h.put((x,y,z),1,strength+curflux,[index])

print 'Saving to', opts.map
h.to_fits(opts.map)




#! /usr/bin/env python
"""
Removes stable crosstalk from PAPER data.  Uses manually set parameters to
filter delays and fringes that are unlikely to correspond to celestial
sources.  Has same shortcomings as xtalk2.py, and doesn't do as good a
job of customizing filters to antenna distribution, but requires less a priori
knowledge of array.

Author: Aaron Parsons
"""

import aipy as a, numpy as n, os, sys, optparse

o = optparse.OptionParser()
o.set_usage('xtalk1.py [options] *.uv')
o.set_description(__doc__)
o.add_option('--sky_max_fringe', dest='sky_max_fringe', 
    type=float, default=.1,
    help='The highest fringe rate (in Hz) expected from celestial sources.')
o.add_option('--cross_fringe_center', dest='cross_fringe_center',
    type=float, default=0,
    help='The center fringe rate for crosstalk.')
o.add_option('--cross_fringe_width', dest='cross_fringe_width',
    type=float, default=0,
    help='The width of the fringe rate for crosstalk.')
o.add_option('--sky_max_delay', dest='sky_max_delay', 
    type=float, default=600,
    help='The highest delay (in ns) expected from celestial sources.')
o.add_option('--cross_delay_center', dest='cross_delay_center', 
    type=float, default=12,
    help='The center delay (in ns) which is corrupted by xtalk.')
o.add_option('--cross_delay_width', dest='cross_delay_width', 
    type=float, default=140,
    help='The width of the delay (in ns) which is corrupted by xtalk.')
opts, args = p.parse_args(sys.argv[1:])

uv = a.miriad.UV(args[0])
inttime,sdf,nchan = uv['inttime'], uv['sdf'], uv['nchan']
bin_dly = 1 / (sdf * nchan)
max_dly_bin = n.round(opts.sky_max_delay / bin_dly)
d_center = n.round(opts.cross_delay_center / bin_dly)
d_width = n.round(opts.cross_delay_width / bin_dly)
d0 = d_center - d_width
d1 = d_center + d_width
print d0, d1, max_dly_bin
assert(d0 * d1 < 0)     # d0 and d1 have to be of opposite signs
assert(max_dly_bin > 0)
del(uv)

# Group files into batches of 4
args.sort()
for g in range(0, len(args), 4):
    phs_off = {}
    cnt = {}
    files = args[g:g+4]
    print 'Processing group', files
    flag = 0
    for uvfile in files:
        uvofile = uvfile+'x'
        if os.path.exists(uvofile):
            print uvofile, 'exists, skipping group.'
            flag = 1
            break
    if flag: continue
    for uvfile in files:
        print 'Reading', uvfile
        uvi = a.miriad.UV(uvfile)
        uvi.select('auto', 0, 0, include=0)
        # Gather all data
        for (uvw,t,(i,j)),d in uvi.all():
            bl = a.miriad.ij2bl(i,j)
            c = nchan - n.sum(d.mask)
            d = n.fft.ifft(d.filled(0))
            try:
                phs_off[bl].append(d)
                cnt[bl] += c
            except(KeyError):
                phs_off[bl] = [d]
                cnt[bl] = c
        del(uvi)

    bin_dly = 1. / (len(phs_off[phs_off.keys()[0]]) * inttime)
    fng_width = n.round(opts.cross_fringe_width / bin_dly)
    max_fng_bin = n.round(opts.sky_max_fringe / bin_dly)
    fng_center = n.round(opts.cross_fringe_center / bin_dly)
    f0 = fng_center - fng_width
    f1 = fng_center + fng_width
    print f0, f1, max_fng_bin
    assert(f0 * f1 >= 0)   # Ensure fw0 and fw1 have the same sign
    assert(max_fng_bin > 0)

    # Average data by channel over entire file
    for bl in phs_off:
        d = n.array(phs_off[bl])
        d = n.fft.fft(d, axis=0)
        d[:,max_dly_bin:-max_dly_bin] = 0
        d[max_fng_bin:-max_fng_bin] = 0
        if f0 == f1:
            d[f0,d0:] = 0
            d[f0,:d1] = 0
        else:
            d[f0:f1,d0:] = 0
            d[f0:f1,:d1] = 0
        d = n.fft.ifft(d, axis=0)
        d = n.fft.fft(d, axis=1)
        phs_off[bl] = d

    cnt = {}
    for bl in phs_off: cnt[bl] = 0

    # Generate a pipe for removing average phase bias from data
    def phs_corr_mfunc(uv, p, d, f):
        uvw,t,(i,j) = p
        bl = a.miriad.ij2bl(i,j)
        if i == j: return p, d, f
        cnt[bl] += 1
        return p, phs_off[bl][cnt[bl],:], f

    for uvfile in files:
        # Apply the pipe to the data
        print 'Working on', uvfile
        uvofile = uvfile+'x'
        uvi = a.miriad.UV(uvfile)
        uvo = a.miriad.UV(uvofile, status='new')
        uvo.init_from_uv(uvi)
        uvo.pipe(uvi, mfunc=phs_corr_mfunc, raw=True,
            append2hist='XTALK1: removed crosstalk\n')



# py2tex.py -- Translate Python source code to % LaTeX
# \LaTeX\ code that can be typeset using the \|py2tex| documentstyle
# option.
# \ESC
# To typeset a Python module called \|foo.py| with py2tex, create a
# \LaTeX\ file along the following lines.
#
# \begin{verbatim}
# %% frame.tex -- wrapper around foo
# \documentstyle[...,py2tex,...]{...}
# ...
# \begin{document}
# ...
# \PythonSource{foo.pt}
# ...
# \end{document}
# \end{verbatim}
# Then give the command
#
# \I{}{1}{\tt\$ py2tex -o foo.pt foo.py}
#
# Finally run \LaTeX\ on the previously constructed wrapper, like this
#
# \I{}{1}{\tt\$ latex frame}
#
# This will give you a \|.dvi| file that you can print in the normal
# way.
#
# Note that normally the comments are interpreted by \LaTeX.  This
# allows for formulae and other fancy stuff.  However, if you don't
# need this, or if you want to typeset programs that were not
# specifically written to be typeset with py2tex, you can leave
# comments uninterpreted by calling the \|py2tex| script with
# the~\|-v| option.  The same effect can be obtained by ending a
# comment with `\|%ASCII|'. It is also possible to switch back to
# interpreted mode by inserting a comment ending in `\|%TeX|' or
# `\|%LaTeX|'.
#
# Here are some guidelines for writing Python code to be typeset using
# py2tex. Each line of Python code is typeset by \LaTeX\ as a paragraph
# where, in case it is broken up into more than one line, all lines
# following the first are indented by one and a half standard
# indentation more than the indentation of the first line. Py2tex does
# not count parentheses to determine whether a line is a continuation
# of the previous or not.  So if you want it to be indented
# appropriately, escape the end of the previous line with a backslash.
# Then py2tex will treat the joined lines as one line, and it will
# inform \LaTeX\ that the escaped line breaks are good points to break
# it up again.  Because \LaTeX\ may decide to break the code at other
# positions (or not at all), these lines will not be numbered.
#
# Consecutive lines that start with a single hash mark (\##) right
# after the indentation are joined and typeset in a \|\vbox| (more
# precise: a \|\vtop|).  This is called a block comment.  Indentation
# changes have no effect within a block comment.  It is possible to
# escape from the \|\vbox| and set the remainder of the block comment
# in what Knuth calls `outer vertical mode' by using the \|\ESC|
# command.  This can be used to incorporate long stretches of \LaTeX\
# code that can spread out over several pages.  Unindented block
# comments are automatically escaped in their entirety.
#
# If a line starts with at least two hash marks it is typeset as if it
# followed some Python code.  The second hash mark also switches
# immediately back to Python mode (see below).  This feature is also
# implemented for {\sc ascii} mode, while the general escape to Python
# mode is not.  (This feature is intended to disable lines of Python
# code by placing two hash marks before them.  This ensures that the
# formatting will be very similar to the uncommented version.)
#
# Comments following Python code are typeset on the same line as the
# Python code, separated from it by a \|\quad| space and the hash
# mark.
#
# Both in block and in line comments the hash mark is used to switch
# between \LaTeX\ and Python mode, just like the dollar sign (\$) is
# used to switch between horizontal and math mode.  This means
# that hash marks are not visible as such in the output.  However, two
# consecutive hash marks are passed to \LaTeX\ as one.  This means that
# it is possible to typeset a hash mark by putting \|\####| in a
# comment.  (This can also be used to define \LaTeX\ macros and to
# include \|\halign| templates, albeit at the expense of doubling all
# hash marks.)  Note that this works only in \LaTeX\ mode, {\em not\/}
# in {\sc ascii} mode.
#
# So if you type
#
# \vskip\parskip\vbox{\parskip=0mm
# \I{}{1}{\tt \## \% LaTeX}
# \I{}{1}{\tt \## Hash mark in comment:\ \|\####|,}
# \I{}{1}{\tt \## formula in comment:\ \$i\_0\char`\\to\char`\\infty\$.}
# \I{}{1}{\tt print chr (i) \## where \##040<=i<=0x7E}
# \I{}{1}{\tt \##\## print #'#'# \## print one hash sign \% ASCII}
# \I{}{1}{\tt \##\## print #i_0*'#'# \## where i\_0 is \##hash signs}
# \par}
#
# you get
#
# \vskip\parskip\vbox{\parskip=0mm
# \B{}{1}{
#    Hash mark in comment: \##,
#    formula in comment: $i_0\to\infty$.
# }
# \I{}{1}#print chr (i)#\quad\## where #040<=i<=0x7E#
# \I{}{1}\## #print '#'#\quad\## print one hash sign
# \I{}{1}\## #print i_0*'#'#\quad\## where i\_0 is \##hash signs
# \par}
#
# Triple quoted strings that occur as the first non-comment after a line
# that ends in a colon (:) are treated as documentation strings.
# There is are three different options for treating
# them. If #docprocess=='none'#, this results in the ``Same 'ol behaviour'':
# \PythonSource*{docstring-none.pt}
# If #docprocess=='plain'#, docstrings are typeset as verbatim
# comments except with thick solid lines instead of thin double
# lines:
# \PythonSource*{docstring-plain.pt}
# If #docprocess=='struct'#, docstrings are typeset as structured text
# as defined by the doc-sig. This is so people can potentially
# write programs that look good both under gendoc and py2tex.
# \PythonSource*{docstring-struct.pt}
#
# It is possible to include the formatted version of another Python
# source file using the \|\PythonSource*| macro.  This was done below
# to give an example of the use of class #Interpret#.
# The starred version of the macro is needed to drop the line numbers,
# otherwise they would be typeset through the lines that mark the
# block comment.  The starred version of \|\PythonSource| also drops
# the section heading. If you escape the block comment (using \|\ESC|)
# you can use the unstarred version again.
#
# \begin{table}\sf
# \hrule height 1mm
# \smallskip
# {\bf Table 1.} Some Python constructs get special typographic treatment
#
# \medskip
# \halign{\tt ##\hfil\qquad&##\hfil\cr
# \omit\bf Python\hfil        &\omit\bf \LaTeX \cr
# =                &#=# \cr
# ==                &#==# \cr
# <=, >=        &#<=, >=# \cr
# !=, <>        &#!=# \cr
# <<, >>        &#<<, >># \cr
# and, or, not        &#and, or, not# \cr
# in, not in        &#in, not in# \cr
# is, is not        &#is, is not# \cr
# }
# \smallskip
# \hrule height 1mm
# \end{table}
#
# Finally some remarks about the formatting of Python
# constructs. Identifiers (keywords, variables and functions) are
# typeset in sans serif.  If an identifier consists of only one
# character, it is typeset in {\em math italic\/} instead of sans
# serif.  Keywords are typeset in boldface, functions (actually:
# identifiers before opening parentheses) are typeset slanted.  These
# typefaces can be changed by redefining some of the macros in
# \|py2tex.sty|.  See the documentation of the style file for
# customization instructions.
#
# Some constructs that get special treatment are listed in Table~1.
# This special treatment is optional.  If the class is initialized
# with an extra argument that evaluates to false, or if the
# #no_math()# method is used, then no special treatment is done for
# these constructs.  (Special treatment can be turned back on half way
# through a file using the #math()# method.)
#
# In strings, characters outside the range #' '#--#'~'# are typeset as
# standard escape sequences (\eg, \|TAB| is typeset as #'        '#,
# \|ESC| is typeset as #''#).  A floating point literal with an
# exponent has its exponent written out as a power of ten (\eg,
# \|3e-6| is typeset as #3e-6#).  Hexadecimal literals are typeset in
# a typewriter font with a lower case \|x| and uppercase digits (\eg,
# \|0X007e| is typeset as #0X007e#).  Octal literals are typeset in
# italics (\eg, \|0377| is typeset as #0377#).

import os, re, string, sys, time

# Usage of class #Interpret#.
#\ESC
# \PythonSource*{example.pt}
# Note that #sys.stdin# is used if #name in (None, '-')#.
#
# The other methods can best be viewed as private to the class.
class Interpret:
    def __init__ (self, name, math = 1, interpret = 1, docprocess='none'):
        if name == None:
            self._name = '-'
        else:
            self._name = name
        if self._name == '-':
            self._name = '(stdin)'
            mtime = time.asctime (\
                  time.localtime (time.time ()))
            self._file = sys.stdin
        else:
            mtime = time.asctime (\
                  time.localtime (os.stat (name) [8]))
            self._file = open (self._name, 'r')
            self._name = os.path.basename (self._name)
        preamble = '\\File{%s}{%s}\n\n' % (self._name, mtime)
        if not math: preamble = preamble + '\\PythonNoMath\n\n'
        self._translation = [preamble,]
        self._docstuff = [preamble,]
        self._math = math
        self._line_nr = 0
        self._line = None
        self._old_line = None
        self._eof = 0
        self._indent_stack = [0]
        self._no_break = 0
        self._interpret_comments = interpret
        self._docprocess = docprocess
        self._docstring = 1
    def math (self):
        if not self._math:
            self._translation.append ('\\PythonMath\n')
            self._docstuff.append ('\\PythonMath\n')
            self._math = 1
    def no_math (self):
        if not self._math:
            self._docstuff.append ('\\PythonNoMath\n')
            self._math = 0
    def interpret (self):
        self._interpret_comments = 1
    def verbatim (self):
        self._interpret_comments = 0
    def close (self):
        self._file.close ()
        self._line_nr = 0
        self._line = None
        self._old_line = None
        self._indent_stack = [0]
        self._translation = []
        self._docstuff = []
        self._eof = 1
        self._no_break = 0
    def flush (self):
        self._file.flush ()
    def next_line (self):
        if self._old_line != None:
            self._line = self._old_line
            self._old_line = None
            self._line_nr = self._line_nr + 1
            return
        self._line = self._file.readline ()
        if self._line == '':
            self._eof = 1
            raise EOFError
        if self._line [-1] == '\n': self._line = self._line [:-1]
        self._line_nr = self._line_nr + 1
    def undo_line (self):
        if self._line != None:
            self._old_line = self._line
            self._line = None
            self._line_nr = self._line_nr - 1
    def close_tex (self, tex):
        while 1:
            if tex [-2:] == '\\ ':
                tex = tex [:-2]
            elif tex [-4:] == '\\BP ':
                tex = tex [:-4]
            else:
                break
        if tex not in ('$', '${}'):
            self._translation.append (tex + '$')
            if tex.find(r'\K{def}') != -1 or tex.find(r'\K{class}') != -1:
                self._docstuff.append(tex + '$')
            elif len(self._docstuff) > 0 and self._docstuff[-1].startswith('\\I{'):
                self._docstuff = self._docstuff[:-1]
    def tr_indentation (self):
        length = white_re.match (self._line)
        if length < 0: raise error
        indent = 0
        for c in self._line [:length]:
            indent = indent + 1
            if c == '\t':
                indent = indent + 8
                indent = indent & ~0x7
        self._line = self._line [length:]
        while indent < self._indent_stack [-1]:
            del self._indent_stack [-1]
        if indent > self._indent_stack [-1]:
            self._indent_stack.append (indent)
        self._indentation = len (self._indent_stack) - 1
    def tr_comment_line (self):
        if self._interpret_comments:
            length = verbatim_re.search (self._line)
            if length >= 0:
                self.verbatim ()
                self._line = self._line [:length]
            while 1:
                hash = string.find (self._line, '#')
                if hash >= 0:
                    if len (self._line) > hash + 1 and \
                          self._line [hash + 1] == '#':
                        self._translation.append (\
                              self._line [:hash] + '#')
                        #self._docstuff.append (\
                        #      self._line [:hash] + '#')
                        self._line = self._line [hash + 2:]
                        continue
                    self._translation.append (self._line [:hash])
                    #self._docstuff.append (self._line [:hash])
                    self._line = self._line [hash + 1:]
                    self.tr_code (0) # No continued lines in comments.
                    if len (self._line) <= 0: break
                    if self._line [0] != '#': raise error
                    self._line = self._line [1:]
                else:
                    break
            self._translation.append (self._line + '\n')
            #self._docstuff.append (self._line + '\n')
        else:
            length = interpret_re.search (self._line)
            if length >= 0:
                self.interpret ()
                self._line = self._line [:length]
            while len (self._line) > 0:
                length = ordinary_re.match (self._line)
                if length > 0:
                    self._translation.append (self._line [:length])
                    #self._docstuff.append (self._line [:length])
                if len (self._line) > length:
                    char = self._line [length]
                    if char in '<>\\{|}~':
                        self._translation.append (\
                              '{\\tt\\char`\\%s}' % char)
                        #self._docstuff.append (\
                        #      '{\\tt\\char`\\%s}' % char)
                    else:
                        self._translation.append ('\\' + char)
                        #self._docstuff.append ('\\' + char)
                self._line = self._line [length+1:]
            self._translation.append ('\n')
            #self._docstuff.append ('\n')
    def tr_block_comment (self):
        if self._line [0] != '#': raise error
        outer = self._indentation == 0
        if outer:
            if self._line_nr > 1:
                self._translation.append (\
                      '\\PythonOuterBlock\n')
                #self._docstuff.append (\
                #      '\\PythonOuterBlock\n')
            else:
                self._translation.append (\
                      '\\PythonOuterBlock*\n')
                #self._docstuff.append (\
                #      '\\PythonOuterBlock*\n')
        else:
            self._translation.append ('\\B{%d}{%d}{%%\n' % \
                  (self._line_nr, self._indentation))
            #self._docstuff.append ('\\B{%d}{%d}{%%\n' % \
            #      (self._line_nr, self._indentation))
        try:
            white = white_re.match (self._line, 1)
            if white < 0: raise error
            self._line = self._line [white:]
            while 1:
                self.tr_comment_line ()
                self.next_line ()
                white = white_re.match (self._line)
                if white < 0: raise error
                if len (self._line) > white and self._line [white] == '#' and \
                      self._line [white:white + 2] != '##':
                    self._line = self._line [white + 1:]
                    white = white_re.match (self._line)
                    if white > 0: self._line = self._line [white:]
                    continue
                self.undo_line ()
                return
        finally:
            if outer:
                self._translation.append (\
                      '\\PythonOuterBlockEnd\n')
                #self._docstuff.append (\
                #      '\\PythonOuterBlockEnd\n')
            else:
                self._translation.append ('}\n')
                #self._docstuff.append ('}\n')
    def tr_comment (self):
        self._translation.append ('\\#\\ ')
        #self._docstuff.append ('\\#\\ ')
        while self._line [:2] == '##':
            self._line = self._line [2:]
            self.tr_code (0) # No continued lines in comments.
            if self._line [:1] == '#':
                self._translation.append ('\\quad\\#\\ ')
                #self._docstuff.append ('\\quad\\#\\ ')
        if len (self._line) < 1:
            self._translation.append ('\n')
            #self._docstuff.append ('\n')
            return
        if self._line [0] != '#': raise error
        white = white_re.match (self._line, 1)
        if white < 0: raise error
        self._line = self._line [white:]
        self.tr_comment_line ()
        return
    def tr_string (self, token):
        quote = token [0]
        tl = len (token)
        self._translation.append ('\S{' + token)
        #self._docstuff.append ('\S{' + token)
        while 1:
            pos = string.find (self._line, quote)
            if pos > 0:
                self._translation.append ('\\verb*%s%s' % \
                      (quote, ctrl_protect (self._line [:pos + 1])))
                #self._docstuff.append ('\\verb*%s%s' % \
                #      (quote, ctrl_protect (self._line [:pos + 1])))
                if escape_re.match (self._line [:pos]) == pos:
                    self._translation.append (quote)
                    #self._docstuff.append (quote)
                    self._line = self._line [pos + 1:]
                    continue
                self._line = self._line [pos:]
                pos = 0
            if pos >= 0:
                if self._line [:tl] == token:
                    self._translation.append (token + '}')
                    #self._docstuff.append (token + '}')
                    self._line = self._line [tl:]
                    return
                self._translation.append (quote)
                #self._docstuff.append (quote)
                self._line = self._line [1:]
            else:
                self._translation.append ('\\verb*%s%s%s' % \
                      (quote, ctrl_protect (self._line), quote))
                #self._docstuff.append ('\\verb*%s%s%s' % \
                #      (quote, ctrl_protect (self._line), quote))
                self._line = ''
                if tl == 1:
                    self._translation.append ('}')
                    #self._docstuff.append ('}')
                    return
                self.next_line ()
                self._translation.append ('\n\\I{%d}{0}' % \
                      self._line_nr)
                #self._docstuff.append ('\n\\I{%d}{0}' % \
                #      self._line_nr)
        return

    def tr_docstring_plain (self):
        length = quote_re.match(self._line)
        token = self._line[:length]
        self._line = self._line[length:]
        quote = token [0]
        tl = len (token)

        if self._indentation == 0:
            self._translation.append ('\\PythonDocBlock\n')
            self._docstuff.append ('\\PythonDocBlock\n')
        else:
            self._translation.append ('\DS{%s}{%s}{%%\n' % 
                                      (self._line_nr, self._indentation))
            self._docstuff.append ('\DS{%s}{%s}{%%\n' % 
                                      (self._line_nr, self._indentation))
        while 1:
            pos = string.find (self._line, quote)
            if pos > 0:
                self._translation.append ('\\verb%s%s' % \
                      (quote, ctrl_protect (self._line [:pos+1])))
                self._docstuff.append ('\\verb%s%s' % \
                      (quote, ctrl_protect (self._line [:pos+1])))
                if escape_re.match (self._line [:pos]) == pos:
                    self._translation.append (quote)
                    self._docstuff.append (quote)
                    self._line = self._line [pos + 1:]
                    continue
                self._line = self._line [pos:]
                pos = 0
            if pos >= 0:
                if self._line [:tl] == token:
                    self._line = self._line [tl:]
                    break
                self._translation.append (quote)
                self._docstuff.append (quote)
                self._line = self._line [1:]
            else:
                self._translation.append ('\\verb%s%s%s' % \
                      (quote, ctrl_protect (self._line), quote))
                self._docstuff.append ('\\verb%s%s%s' % \
                      (quote, ctrl_protect (self._line), quote))
                self._line = ''
                if tl == 1:
                    break
                self.next_line ()
                # XXX This assumes 8 spaces per tab.
                wchars = white_re.match(self._line)
                spaces = re.sub('\t', ' '*8, self._line[:wchars])
                indent = white_re.match(spaces)
                ## print `spaces`, indent, self._indentation
                self._line = spaces[self._indentation*4:] + self._line[wchars:]
                self._translation.append ('\\\\ \n')
                self._docstuff.append ('\\\\ \n')
        if self._indentation == 0:
            self._translation.append ('\n\\PythonDocBlockEnd\n')
            self._docstuff.append ('\n\\PythonDocBlockEnd\n')
        else:
            self._translation.append ('}\n')
            self._docstuff.append ('}\\vskip 5pt \n')

    def tr_docstring_struct (self):
        length = quote_re.match(self._line)
        token = self._line[:length]
        self._line = self._line[length:]
        quote = token [0]
        tl = len (token)
        if self._indentation == 0:
            self._translation.append ('\\PythonDocBlock\n')
            self._docstuff.append ('\\PythonDocBlock\n')
        else:
            self._translation.append ('\DS{%s}{%s}{%%\n' % 
                                      (self._line_nr, self._indentation))
            self._docstuff.append ('\DS{%s}{%s}{%%\n' % 
                                      (self._line_nr, self._indentation))
        docstring = []
        while 1:
            pos = string.find (self._line, quote)
            if pos > 0:
                docstring.append (self._line [:pos])
                if escape_re.match (self._line [:pos]) == pos:
                    docstring.append (quote)
                    self._line = self._line [pos+1:]
                    continue
                self._line = self._line [pos:]
                pos = 0
            if pos >= 0:
                if self._line [:tl] == token:
                    self._line = self._line [tl:]
                    break
                docstring.append (quote)
                self._line = self._line [1:]
            else:
                docstring.append (self._line)
                self._line = ''
                if tl == 1:
                    break
                self.next_line ()
                docstring.append ('\n')
        docstring = string.joinfields(docstring, '')
        import struct2latex
        structstring = str(struct2latex.LaTeX(docstring))
        if self._indentation == 0:
            self._translation.append ('%s\n\\PythonDocBlockEnd\n' % structstring)
            self._docstuff.append ('%s\n\\PythonDocBlockEnd\n' % structstring)
        else:
            self._translation.append ('%s}' % structstring)            
            self._docstuff.append ('%s}' % structstring)            

    def tr_code (self, allow_continue = 1):
        tex = '$'
        try:
            careful   = 0
            while 1:
                white = white_re.match (self._line)
                if white > 0:
                    self._line = self._line [white:]
                if len (self._line) <= 0: return
                if self._line == '\\':
                    if allow_continue:
                        tex = tex + '\\BP '
                        self.next_line ()
                        continue
                    else:
                        self._line = ''
                        return
                if self._line [0] == '#': return
                length = token_re.match (self._line)
                if length < 1:
                    length = numeral_re.match (self._line)
                    if length < 1:
                        tex = tex + self._line [0]
                        self._line = self._line [1:]
                        careful = 0
                    else:
                        token = self._line [:length]
                        self._line = self._line [length:]
                        if careful: tex = tex + '\\ '
                        tex = tex + tr_numeral (token)
                        careful = 1
                    continue
                token = self._line [:length]
                self._line = self._line [length:]
                token = self.double (token)
                if token == ':':
                    self._docstring = 1
                else:
                    self._docstring = 0
                if token in ('{', '}'):
                    tex = tex + '\\' + token
                    careful = 0
                    continue
                if token in reserved_operators:
                    tex = tex + '\\O{%s}' % token
                    careful = 0
                    continue
                if token [0] in string.letters + '_':
                    if careful: tex = tex + '\\ '
                    new_careful = 1
                    if token in reserved:
                        if tex [-2:] not in ('$', '\\ ') and not careful:
                            tex = tex + '\\ '
                        tex = tex + '\\K{%s}' % token
                        if token == 'if': tex = tex + '\\,'
                        if token not in single: tex = tex + '\\ '
                        new_careful = 0
                    else:
                        token = usc_protect (token)
                        length = function_re.match (self._line)
                        if length > 0:
                            self._line = self._line [length:]
                            tex = tex + '\\F{%s}\\,(' % token
                            new_careful = 0
                        else:
                            if len (token) == 1:
                                tex = tex + token
                            else:
                                tex = tex + '\\V{%s}' % token
                    careful = new_careful
                    continue
                if token [0] in '\'"':
                    self.close_tex (tex + '{}')
                    self.tr_string (token)
                    tex = '${}'
                    careful = 0
                    continue
                if '{' in token or '}' in token:
                    raise ValueError, "brace in token '%s'" % token
                tex = tex + '\\Y{%s}' % token
                careful = 0
        finally:
            self.close_tex (tex)
    def double (self, token):
        if token not in ('not', 'is'): return token
        white = white_re.match (self._line)
        if white > 0:
            self._line = self._line [white:]
        next_length = token_re.match (self._line)
        if next_length > 0:
            next = self._line [:next_length]
            if (token, next) in (('not', 'in'), \
                  ('is', 'not')):
                self._line = self._line [next_length:]
                return token + ' ' + next
        return token
    # Method #translate()# is the interface to the #Interpret# class.
    # It calls the $\F{tr\_}xxx()$ methods to process indentation, code,
    # comments and strings.
    def translate (self):
        self._docstuff = []
        self._translation = []
        if self._eof: return None
        try:
            empty = 0
            self.next_line ()
            while white_re.match (self._line) == len (self._line):
                empty = empty + 1
                self.next_line ()
            if empty > 0:
                self._translation.append ('\\E{%d}' % empty)
                #self._docstuff.append ('\\E{%d}' % empty)
            self.tr_indentation ()
            if len (self._line) > 0 and \
                  self._line [0] == '#' and self._line [:2] != '##':
                self.tr_block_comment ()
                self._no_break = 1
            elif self._docprocess != 'none' and \
                 self._docstring and self._line[:3] in ('"""', "'''"):
                if self._docprocess == 'plain':
                    self.tr_docstring_plain()
                elif self._docprocess == 'struct':
                    self.tr_docstring_struct()
                else:
                    raise ValueError, 'Illegal value for doc process.'
            else:
                self._translation.append ('\\I{%d}{%d}' % \
                      (self._line_nr, self._indentation))
                self._docstuff.append ('\\I{%d}{%d}' % \
                      (self._line_nr, self._indentation))
                self.tr_code ()
                if not self._no_break and \
                      self._translation [-1] [-8:] == '\\colon $' and \
                      self._translation [0] [:3] != '\\E{':
                    self._translation.insert (0, '\\PB')
                    self._docstuff.insert (0, '\\PB')
                    self._no_break = 1
                else:
                    self._no_break = empty != 0
                if len (self._line) > 0:
                    if self._line [:1] != '#': raise error
                    if self._translation [-1] [:1] == '$':
                        self._translation.append ('\\quad ')
                        #self._docstuff.append ('\\quad ')
                    self.tr_comment ()
                else:
                    self._translation.append ('\n')
                    self._docstuff.append ('\n')
        except EOFError: pass
        return self._translation
    def translation (self):
        return self._docstuff
        #return self._translation

error = 'py2tex error'

class Re:
        def __init__(self, regex):
                self._regex = regex
        def match(self, string, pos = 0):
                m = self._regex.match(string, pos)
                result = -1
                if m:
                        result = m.end(0)
                return result
        def search(self, string, pos = 0):
                m = self._regex.search(string, pos)
                result = -1
                if m:
                        result = m.start(0)
                return result

class Regex:
        def compile(self, regex):
                return Re(re.compile(regex))

regex = Regex()

interpret_re = regex.compile ('%[ \t]*(La)?TeX[ \t]*$')
verbatim_re = regex.compile ('%[ \t]*ASCII[ \t]*$')
ordinary_re = regex.compile ('[^#$%&<>\\\\^_{|}~]*')
white_re = regex.compile ('[ \t]*')
function_re = regex.compile ('[ \t]*\\(')
comment_re = regex.compile ('(##|[^#])*')
escape_re = regex.compile ('([^\\\\]|\\\\.)*\\\\')
numeral_re = regex.compile (string.joinfields ((
      '0[xX][0-9A-Fa-f]+',
      '[0-9]+\\.?[eE][+-]?[0-9]+[jJLl]?',
      '[0-9]*\\.[0-9]+[eE][+-]?[0-9]+[jJLl]?',
      '[1-9][0-9]*[jJLl]?',
      '0[0-7]*'), '|'))

token_re = regex.compile (string.joinfields ((
      '[A-Za-z_][A-Za-z_0-9]*',
      "'('')?", '"("")?',
      '==?', '[<>!]=', '<>',
      '<<', '>>',
      '\\[]',
      '[*][*]',
      '[\\\\{}$&|^~%:*/+-]'), '|'))
quote_re= regex.compile( '("("")?)|' "('('')?)" ) 

TeX_code = {
      '\\': '$\\backslash$', '|': '$\\vert$',
      '<': '$<$', '>': '$>$',
      '{': '$\\{$', '}': '$\\}$' }
reserved = ('access', 'and', 'break', 'class', 'continue',
      'def', 'del', 'elif', 'else', 'except', 'exec',
      'finally', 'for', 'from', 'global', 'if',
      'import', 'in', 'is', 'is not', 'not', 'not in', 'or',
      'pass', 'print', 'raise', 'return', 'try', 'while')
single = ('else', 'finally', 'try', '-', '+')
reserved_operators = ('and', 'in', 'is', 'is not', 'not', 'not in', 'or', '**')
special_ctrl = { '\a': '\\a', '\b': '\\b', '\f': '\\f',
      '\n': '\\n', '\r': '\\r', '\t': '\\t', '\v': '\\v' }

def usc_protect (ident):
    ident = string.joinfields (string.splitfields (ident, '_'), '\\_')
    return ident

def ctrl_protect (str):
    result = ''
    for c in str:
        o = ord (c)
        if o < 32 or o >= 127:
            if special_ctrl.has_key (c):
                result = result + special_ctrl [c]
            else:
                result = '%s\\%03o' % (result, o)
        else:
            result = result + c
    return result

def tr_numeral (token):
    end = token[-1]                        # Preserve the type signifier (jJlL) if any.
    numeral = string.lower (token)
    if numeral [:2] == '0x':
        # #(0x1a, 0X2B)
        return '\\HEX{%s}' % string.upper (numeral [2:])
    if not (end in'jJlL'):                # Check if end is a signifier.
        end = ''
    else:
        numeral = numeral[:-1]                # Strip the signifier.
    pos = string.find (numeral, 'e')
    if pos >= 0:
        # #(12.4e-78, .3333E+0, .1e6, 2.e1, 0.e1, 1e4)
        return '\\EXP{%s}{%s}{%s}' % \
              (numeral [:pos], numeral [pos + 1:], end)
    if numeral [:1] == '0' and numeral != '0':
        # #(0377, 0378)
        return '\\OCT{%s}' % numeral [1:]
    # #(.333, 3.141592)# #(0, 1, 42)
    return '\\NUM{%s}{%s}' % (numeral, end) 



#! /usr/bin/env python
import aipy._healpix, numpy

hp = aipy._healpix.HealpixBase()
print hp.order(), hp.nside(), hp.npix(), hp.scheme()
hp = aipy._healpix.HealpixBase(2**6)
print hp.order(), hp.nside(), hp.npix(), hp.scheme()
hp = aipy._healpix.HealpixBase(2**6, 'NEST')
print hp.order(), hp.nside(), hp.npix(), hp.scheme()
assert(hp.npix2nside(hp.npix()) == hp.nside())
px = numpy.arange(hp.npix())
px = hp.nest_ring_conv(px, 'RING')
hp.set_nside_scheme(hp.nside(), 'RING')
px = numpy.arange(10)
x,y,z = hp.px2crd(px, ncrd=3)
th,phi = hp.px2crd(px, ncrd=2)
assert(numpy.all(px == hp.crd2px(x,y,z, interpolate=0)))
px2,wgt = hp.crd2px(x,y,z, interpolate=1)
z[-1] = numpy.Inf
try: hp.crd2px(x,y,z[:-1])
except(RuntimeError): pass
assert(numpy.all(px == hp.crd2px(th, phi, interpolate=0)))
px2,wgt = hp.crd2px(th, phi, interpolate=1)



"""Load an AntennaArray using the calibration information contained in the
specified location-specific module."""

import numpy as n, ant, sim, fit, os, sys, coord

def get_freqs(sdf, sfreq, nchan):
    return n.arange(nchan, dtype=n.float) * sdf + sfreq

def get_aa(*args):
    '''Return the AntennaArray specified by loc_key, which should be the
    name of a module somewhere in Python's path that implements a 
    "get_aa(freqs)" function.  That function should return an AntennaArray 
    initialized with appropriate calibration parameters.  This function simply 
    attempts to import that function, and then performs a simple conversion 
    between sdf (channel width in GHz), sfreq (starting frequency), nchan (#
    of channels) --> freqs (an array of channel centers).'''
    if len(args) == 2: loc_key,freqs = args
    else:
        loc_key,sdf,sfreq,nchan = args
        freqs = get_freqs(sdf, sfreq, nchan)
    sys.path.append(os.getcwd())
    exec('from %s import get_aa as _get_aa' % loc_key)
    return _get_aa(freqs)

def get_src_prms(loc_key):
    sys.path.append(os.getcwd())
    try: exec('from %s import src_prms' % loc_key)
    except(ImportError): src_prms = {}
    return src_prms



#! /usr/bin/env python
"""
A script for dividing out the passband, primary beam, and/or source spectrum 
scaling.  When dividing by a primary beam or source spectrum, it is recommended
a single source have been isolated in the data set.

Author: Aaron Parsons
"""

import aipy as a, numpy as n, pylab as p, os, sys, optparse, pickle

o = optparse.OptionParser()
o.set_usage('flux_cal.py [options] *.uv')
o.set_description(__doc__)
a.scripting.add_standard_options(o, src=True, loc=True)
o.add_option('-b', '--beam', dest='beam', action='store_true',
    help='Normalize by the primary beam response in the direction of the specified source.')
o.add_option('-p', '--passband', dest='passband', action='store_true',
    help='Normalize by the passband response.')
o.add_option('-f', '--srcflux', dest='srcflux', action='store_true',
    help='Normalize by the spectrum of the specified source.')
opts, args = o.parse_args(sys.argv[1:])

assert(not (opts.src is None and (opts.beam or opts.srcflux)))
uv = a.miriad.UV(args[0])
freqs = a.loc.get_freqs(uv['sdf'], uv['sfreq'], uv['nchan'])
aa = a.loc.get_aa(opts.loc, uv['sdf'], uv['sfreq'], uv['nchan'])
del(uv)

if opts.srcflux:
    src = a.scripting.parse_srcs(opts.src, force_cat=True)
    src.set_params(a.loc.get_src_prms(opts.loc))
    s = src.values()[0]
    print 'Calibrating for source with',
    print 'strength', s._janskies,
    print 'measured at', s.mfreq, 'GHz',
    print 'with index', s._index
    src_spec = None
else: src_spec = 1

curtime = None
def mfunc(uv, p, d, f):
    global curtime, src_spec
    uvw,t,(i,j) = p
    pol = a.miriad.pol2str[uv['pol']]
    if t != curtime:
        curtime = t
        aa.set_jultime(t)
        if opts.srcflux:
            src.compute(aa)
            src_spec = src.get_fluxes()
            s_eq = src.get_crds('eq', ncrd=3)
            aa.sim_cache(s_eq)
    if i == j: return p, d, f
    if opts.passband: passband = aa.passband(i,j)
    else: passband = 1
    if opts.beam: bm_resp = aa.bm_response(i,j,pol=pol).squeeze()
    else: bm_resp = 1
    gain = passband * bm_resp * src_spec
    d /= n.where(gain == 0, 1, gain)
    return p, d, f

ext = '.'
if opts.passband: ext += 'p'
if opts.beam: ext += 'b'
if opts.srcflux: ext += 'f'
for filename in args:
    print filename
    uvofile = filename + ext
    if os.path.exists(uvofile):
        print 'File exists: skipping'
        continue
    uvi = a.miriad.UV(filename)
    uvo = a.miriad.UV(uvofile, status='new')
    uvo.init_from_uv(uvi)
    uvo.pipe(uvi, mfunc=mfunc, raw=True, 
        append2hist='FLUXCAL: srcs=%s passband=%s beam=%s srcflux=%s\n' % \
        (opts.src, opts.passband, opts.beam, opts.srcflux))



#!/usr/bin/python
"""
Detect and flag RFI related effects in UV files.  Uses statistical thresholding
to identify outliers in power and narrowness of features.  Does an improved
job of identifying low-level interference if sky model and crosstalk are
removed from the data first.

Author: Aaron Parsons
"""

import numpy as n, aipy as a, os, sys, pickle, optparse

o = optparse.OptionParser()
o.set_usage('xrfi.py [options] *.uv')
o.set_description(__doc__)
o.add_option('-c', '--chan', dest='chan', 
    help='Manually flag channels before xrfi processing.  Options are "<chan1 #>,..." (a list of channels), or "<chan1 #>_<chan2 #>" (a range of channels).  Default is None.')
o.add_option('-n', '--nsig', dest='nsig', default=2., type='float',
    help='Number of standard deviations above mean to flag.  Default 2.')
o.add_option('-m', '--flagmode', dest='flagmode', default='both',
    help='Can be val,int,both for flagging by value only, integration only, or both.  Default both.')
o.add_option('--ch_thresh', dest='ch_thresh',type='float',default=.33,
    help='Fraction of the data in a channel which, if flagged, will result in the entire channel being flagged.  Default .33')
o.add_option('--int_thresh', dest='int_thresh',type='float',default=.66,
    help='Fraction of the data in an integration which, if flagged, will result in the entire integration being flagged.  Default .33')
o.add_option('-i', '--infile', dest='infile', action='store_true',
    help='Apply xrfi flags generated with the -o option.')
o.add_option('-o', '--outfile', dest='outfile', action='store_true',
    help='Rather than apply the flagging to the data, store them in a file (named by JD) to apply to a different file with the same JD.')
opts, args = o.parse_args(sys.argv[1:])

# Parse command-line options
uv = a.miriad.UV(args[0])
if not opts.chan is None:
    chans = a.scripting.parse_chans(opts.chan, uv['nchan'])
else:
    chans = []
    opts.chan = 'None'
del(uv)

for uvfile in args:
    print 'Working on', uvfile
    uvofile = uvfile+'r'
    if os.path.exists(uvofile):
        print uvofile, 'exists, skipping.'
        continue
    uvi = a.miriad.UV(uvfile)
    (uvw,jd,(i,j)),d,f = uvi.read(raw=True)
    uvi.rewind()
    if opts.infile:
        if not os.path.exists('%f.xrfi' % jd):
            print '%f.xrfi' % jd, 'does not exist.  Skipping...'
            continue
        f = open('%f.xrfi' % jd)
        mask = pickle.load(f)
        f.close()
        for m in mask: mask[m] = n.array(mask[m])
    else:
        # Gather all data and each time step
        window = None
        data,mask,times = {}, {}, []
        for (uvw,t,(i,j)), d, f in uvi.all(raw=True):
            if len(times) == 0 or times[-1] != t: times.append(t)
            mask[t] = mask.get(t, 0) | f
            bl = a.miriad.ij2bl(i,j)
            if i != j:
                if window is None: 
                    window = d.size/2 - abs(n.arange(d.size) - d.size/2)
                d = n.fft.fft(n.fft.ifft(d) * window)
            pol = uvi['pol']
            if not pol in data: data[pol] = {}
            if not bl in data[pol]: data[pol][bl] = {}
            data[pol][bl][t] = d

        # Generate a single mask for all baselines which masks if any
        # baseline has an outlier at that freq/time.  Data flagged
        # strictly on basis of nsigma above mean.
        if not opts.flagmode.startswith('int'):
            new_mask = {}
            for bl in mask:
                mask[bl][chans] = 1
                new_mask[bl] = mask[bl].copy()
            for pol in data:
              for bl in data[pol]:
                i, j = a.miriad.bl2ij(bl)
                if i == j: continue
                data_times = data[pol][bl].keys()
                d = n.ma.array([data[pol][bl][t] for t in data_times],
                    mask=[mask[t] for t in data_times])
                hi_thr, lo_thr = a.rfi.gen_rfi_thresh(d, nsig=opts.nsig)
                m = n.where(n.abs(d) > hi_thr,1,0)
                for i, t in enumerate(data_times): new_mask[t] |= m[i]
            mask = new_mask
            # If more than ch_thresh of the data in a channel or 
            # integration is flagged, flag the whole thing
            ch_cnt = n.array([mask[t] for t in data_times]).sum(axis=0)
            ch_msk = n.where(ch_cnt > ch_cnt.max()*opts.ch_thresh,1,0)
            for t in mask:
                if mask[t].sum() > mask[t].size * opts.int_thresh:
                    mask[t] |= 1
                else:
                    mask[t] |= ch_msk

        # Use autocorrelations to flag entire integrations which have
        # anomalous powers.  At least 2 antennas must agree for a 
        # integration to get flagged.
        if not opts.flagmode.startswith('val'):
            new_mask = {}
            for pol in data:
              for bl in data[pol]:
                i, j = a.miriad.bl2ij(bl)
                if i != j: continue
                data_times = data[pol][bl].keys()
                data_times.sort()
                d = n.ma.array([data[pol][bl][t] for t in data_times],
                    mask=[mask[t] for t in data_times])
                for i in n.where(a.rfi.flag_by_int(d))[0]:
                    t = data_times[i]
                    new_mask[t] = new_mask.get(t, 0) + 1
            for t in new_mask:
                if new_mask[t] > 1: mask[t] |= 1
    if opts.outfile:
        for t in mask: mask[t] = list(mask[t])
        print 'Writing %f.xrfi' % jd
        f = open('%f.xrfi' % jd, 'w')
        pickle.dump(mask, f)
        f.close()
    else:
        # Generate a pipe for applying both the total_mask and the int_mask
        # to the data as it comes it.
        def rfi_mfunc(uv, preamble, data, flags):
            uvw, t, (i,j) = preamble
            return preamble, data, mask[t]

        uvi.rewind()
        uvo = a.miriad.UV(uvofile, status='new')
        uvo.init_from_uv(uvi)
        uvo.pipe(uvi, mfunc=rfi_mfunc, raw=True, append2hist='XRFI: nsig %f, chans %s, mode %s, ch_thresh %f\n' %  (opts.nsig, opts.chan, opts.flagmode, opts.ch_thresh))



#! /usr/bin/env python

import aipy as a, numpy as n, pylab as p

SIZE = 100
NOISE = .001
i = n.zeros((SIZE,SIZE), n.float)
i[10,10] = 10.
i[20:25,20:25] = 1.
i[30:40,30:40] = .1
b = a.img.gaussian_beam(2, shape=i.shape)
b[0,0] = .05

d = a.img.convolve2d(i, b)
ns = n.random.normal(scale=NOISE, size=i.shape)
d = n.abs(d + ns)

print 'Clean'
p.subplot(221)
c,info = a.deconv.clean(d, b, verbose=True)
p.title('CLEAN')
p.imshow(n.log10(c), vmin=-5, vmax=1)

print 'LSQ'
p.subplot(222)
c,info = a.deconv.lsq(d, b, verbose=True)
p.title('LSQ')
p.imshow(n.log10(c), vmin=-5, vmax=1)

print 'MEM'
p.subplot(223)
c,info = a.deconv.maxent(d, b, n.var(d**2)*.5, verbose=True)
p.title('MEM')
p.imshow(n.log10(c), vmin=-5, vmax=1)

print 'Anneal'
p.subplot(224)
c,info = a.deconv.anneal(d, b, verbose=True)
p.title('Anneal')
p.imshow(n.log10(c), vmin=-5, vmax=1)
p.colorbar()

p.show()




#! /usr/bin/env python
import aipy as a, numpy as n, pylab as p

img = n.array([0,0,0,4,6,4,0,0,-2,-3,-2,0], dtype=n.float)
ker = n.array([3,2,0,0,0,0,0,0,0,0,0,2], dtype=n.float)

print 'REAL TEST:'
print 'img', img
print 'ker', ker
cln, info = a.deconv.clean1d(img, ker)
print 'cln', cln
print info
print '-----------------------------------------------------------------'

img = n.array([0,0,0,4j,6j,4j,0,0,2+2j,3+3j,2+2j,0], dtype=n.complex)
ker = n.array([3,2,0,0,0,0,0,0,0,0,0,2], dtype=n.complex)

print 'CMPLX TEST:'
print 'img', img
print 'ker', ker
mdl, info = a.deconv.clean1d(img, ker)
print 'cln', cln
print info
print '-----------------------------------------------------------------'

SIZE = 16
img = n.zeros((SIZE,), n.complex)
img[5] = 1+2j
img[10] = 3+4j
ker = n.zeros((SIZE,), n.complex)
ker[0] = 2j
ker[1] = 1+1j; ker[-1] = -1-1j
ker[2] = .5+.5j; ker[-2] = -.5-.5j
d1d = n.fft.ifft(n.fft.fft(img) * n.fft.fft(ker))
print 'TEST:'
print 'img', d1d
print 'ker', ker
cln, info = a.deconv.clean1d(d1d, ker, maxiter=200)
print 'cln', cln
print info

p.plot(n.abs(cln), 'b')
p.plot(n.abs(img), 'k.')
p.plot(n.abs(img - cln), 'r')
p.show()



#! /usr/bin/env python
"""Tarball and compress (using bz2) Miriad UV files.
Author: Aaron Parsons
Date: 8/14/07"""

import sys, os
from optparse import OptionParser

p = OptionParser()
p.set_usage('compress_uv.py [options] *.uv')
p.set_description(__doc__)
p.add_option('-d', '--delete', dest='delete', action='store_true',
    help='Delete a uv file after compressing it')
p.add_option('-x', '--expand', dest='expand', action='store_true',
    help='Inflate tar.bz2 files')

opts, args = p.parse_args(sys.argv[1:])

for i in args:
    print i
    if opts.expand:
        rv = os.system('tar xjf %s' % i)
        if rv != 0: break
        continue
    cmp_name = i + '.tar.bz2'
    if os.path.exists(cmp_name):
        print cmp_name, 'exists; skipping...'
        continue
    rv = os.system('tar jcf %s %s' % (cmp_name, i))
    # If tar fails, delete malformed tarball and then exit
    if rv != 0:
        os.system('rm -rf %s' % (cmp_name))
        break
    if opts.delete: os.system('rm -rf %s' % (i))



#! /usr/bin/env python
import aipy as a, numpy as n

freqs = n.arange(1.,2.,.1)
loc = ('0:00', '0:00')
beam = a.ant.Beam(freqs)
ants = [a.ant.Antenna(0,0,0,beam), a.ant.Antenna(100,0,0,beam)]
aa = a.ant.AntennaArray(ants=ants, location=loc)
aa.set_jultime(2454490)
src = a.ant.RadioFixedBody('0','0')
src.compute(aa)
print 'RA=%s' % src.ra, 'DEC=%s' % src.dec
phs = aa.gen_phs(src, 0, 1)
seq1 = a.coord.radec2eq((src.ra, src.dec))
print 'VEC=%s' % seq1
assert(n.all(aa.gen_phs(seq1, 0, 1) == phs))
seq2 = n.array([seq1] * 4)
for p in aa.gen_phs(seq2, 0, 1): assert(n.all(p == phs))
print 'Phases correct'
print 'Passed.'



# Original Author: Travis Oliphant 2002
# Bug-fixes in 2006 by Tim Leslie


import numpy
from numpy import asarray, tan, exp, ones, squeeze, sign, \
     all, log, sqrt, pi, shape, array, minimum, where
from numpy import random

__all__ = ['anneal']

_double_max = numpy.finfo(float).max
class base_schedule(object):
    def __init__(self):
        self.dwell = 20
        self.learn_rate = 0.5
        self.lower = -10
        self.upper = 10
        self.Ninit = 50
        self.accepted = 0
        self.tests = 0
        self.feval = 0
        self.k = 0
        self.T = None

    def init(self, **options):
        self.__dict__.update(options)
        self.lower = asarray(self.lower)
        self.lower = where(self.lower == numpy.NINF, -_double_max, self.lower)
        self.upper = asarray(self.upper)
        self.upper = where(self.upper == numpy.PINF, _double_max, self.upper)
        self.k = 0
        self.accepted = 0
        self.feval = 0
        self.tests = 0

    def getstart_temp(self, best_state):
        assert(not self.dims is None)
        lrange = self.lower
        urange = self.upper
        fmax = -300e8
        fmin = 300e8
        for _ in range(self.Ninit):
            x0 = random.uniform(size=self.dims)*(urange-lrange) + lrange
            fval = self.func(x0, *self.args)
            self.feval += 1
            if fval > fmax:
                fmax = fval
            if fval < fmin:
                fmin = fval
                best_state.cost = fval
                best_state.x = array(x0)
        self.T0 = (fmax-fmin)*1.5
        return best_state.x

    def accept_test(self, dE):
        T = self.T
        self.tests += 1
        if dE < 0:
            self.accepted += 1
            return 1
        p = exp(-dE*1.0/self.boltzmann/T)
        if (p > random.uniform(0.0, 1.0)):
            self.accepted += 1
            return 1
        return 0

    def update_guess(self, x0):
        pass

    def update_temp(self, x0):
        pass


#  A schedule due to Lester Ingber
class fast_sa(base_schedule):
    def init(self, **options):
        self.__dict__.update(options)
        if self.m is None:
            self.m = 1.0
        if self.n is None:
            self.n = 1.0
        self.c = self.m * exp(-self.n * self.quench)

    def update_guess(self, x0):
        x0 = asarray(x0)
        u = squeeze(random.uniform(0.0, 1.0, size=self.dims))
        T = self.T
        y = sign(u-0.5)*T*((1+1.0/T)**abs(2*u-1)-1.0)
        xc = y*(self.upper - self.lower)
        xnew = x0 + xc
        return xnew

    def update_temp(self):
        self.T = self.T0*exp(-self.c * self.k**(self.quench))
        self.k += 1
        return

class cauchy_sa(base_schedule):
    def update_guess(self, x0):
        x0 = asarray(x0)
        numbers = squeeze(random.uniform(-pi/2, pi/2, size=self.dims))
        xc = self.learn_rate * self.T * tan(numbers)
        xnew = x0 + xc
        return xnew

    def update_temp(self):
        self.T = self.T0/(1+self.k)
        self.k += 1
        return

class boltzmann_sa(base_schedule):
    def update_guess(self, x0):
        std = minimum(sqrt(self.T)*ones(self.dims), (self.upper-self.lower)/3.0/self.learn_rate)
        x0 = asarray(x0)
        xc = squeeze(random.normal(0, 1.0, size=self.dims))

        xnew = x0 + xc*std*self.learn_rate
        return xnew

    def update_temp(self):
        self.k += 1
        self.T = self.T0 / log(self.k+1.0)
        return

class _state(object):
    def __init__(self):
        self.x = None
        self.cost = None

# TODO:
#     allow for general annealing temperature profile
#     in that case use update given by alpha and omega and
#     variation of all previous updates and temperature?

# Simulated annealing

def anneal(func, x0, args=(), schedule='fast', full_output=0,
           T0=None, Tf=1e-12, maxeval=None, maxaccept=None, maxiter=400,
           boltzmann=1.0, learn_rate=0.5, feps=1e-6, quench=1.0, m=1.0, n=1.0,
           lower=-100, upper=100, dwell=50):
    """Minimize a function using simulated annealing.

    Schedule is a schedule class implementing the annealing schedule.
    Available ones are 'fast', 'cauchy', 'boltzmann'

    Inputs:

    func         -- Function to be optimized
    x0           -- Parameters to be optimized over
    args         -- Extra parameters to function
    schedule     -- Annealing schedule to use (a class)
    full_output  -- Return optional outputs
    T0           -- Initial Temperature (estimated as 1.2 times the largest
                    cost-function deviation over random points in the range)
    Tf           -- Final goal temperature
    maxeval      -- Maximum function evaluations
    maxaccept    -- Maximum changes to accept
    maxiter      -- Maximum cooling iterations
    learn_rate   -- scale constant for adjusting guesses
    boltzmann    -- Boltzmann constant in acceptance test
                     (increase for less stringent test at each temperature).
    feps         -- Stopping relative error tolerance for the function value in
                     last four coolings.
    quench, m, n -- Parameters to alter fast_sa schedule
    lower, upper -- lower and upper bounds on x0 (scalar or array).
    dwell        -- The number of times to search the space at each temperature.

    Outputs: (xmin, {Jmin, T, feval, iters, accept,} retval)

    xmin -- Point giving smallest value found
    retval -- Flag indicating stopping condition:
                0 : Cooled to global optimum
                1 : Cooled to final temperature
                2 : Maximum function evaluations
                3 : Maximum cooling iterations reached
                4 : Maximum accepted query locations reached

    Jmin  -- Minimum value of function found
    T     -- final temperature
    feval -- Number of function evaluations
    iters  -- Number of cooling iterations
    accept -- Number of tests accepted.

    See also:

      fmin, fmin_powell, fmin_cg,
             fmin_bfgs, fmin_ncg -- multivariate local optimizers
      leastsq -- nonlinear least squares minimizer

      fmin_l_bfgs_b, fmin_tnc,
             fmin_cobyla -- constrained multivariate optimizers

      anneal, brute -- global optimizers

      fminbound, brent, golden, bracket -- local scalar minimizers

      fsolve -- n-dimenstional root-finding

      brentq, brenth, ridder, bisect, newton -- one-dimensional root-finding

      fixed_point -- scalar fixed-point finder

    """
    x0 = asarray(x0)
    lower = asarray(lower)
    upper = asarray(upper)

    schedule = eval(schedule+'_sa()')
    #   initialize the schedule
    schedule.init(dims=shape(x0),func=func,args=args,boltzmann=boltzmann,T0=T0,
                  learn_rate=learn_rate, lower=lower, upper=upper,
                  m=m, n=n, quench=quench, dwell=dwell)

    current_state, last_state, best_state = _state(), _state(), _state()
    if T0 is None:
        x0 = schedule.getstart_temp(best_state)
    else:
        best_state.x = None
        best_state.cost = 300e8

    last_state.x = asarray(x0).copy()
    fval = func(x0,*args)
    schedule.feval += 1
    last_state.cost = fval
    if last_state.cost < best_state.cost:
        best_state.cost = fval
        best_state.x = asarray(x0).copy()
    schedule.T = schedule.T0
    fqueue = [100, 300, 500, 700]
    iters = 0
    while 1:
        for n in range(dwell):
            current_state.x = schedule.update_guess(last_state.x)
            current_state.cost = func(current_state.x,*args)
            schedule.feval += 1

            dE = current_state.cost - last_state.cost
            if schedule.accept_test(dE):
                last_state.x = current_state.x.copy()
                last_state.cost = current_state.cost
                if last_state.cost < best_state.cost:
                    best_state.x = last_state.x.copy()
                    best_state.cost = last_state.cost
        schedule.update_temp()
        iters += 1
        # Stopping conditions
        # 0) last saved values of f from each cooling step
        #     are all very similar (effectively cooled)
        # 1) Tf is set and we are below it
        # 2) maxeval is set and we are past it
        # 3) maxiter is set and we are past it
        # 4) maxaccept is set and we are past it

        fqueue.append(squeeze(last_state.cost))
        fqueue.pop(0)
        af = asarray(fqueue)*1.0
        if all(abs((af-af[0])/af[0]) < feps):
            retval = 0
            if abs(af[-1]-best_state.cost) > feps*10:
                retval = 5
                print "Warning: Cooled to %f at %s but this is not" \
                      % (squeeze(last_state.cost), str(squeeze(last_state.x))) \
                      + " the smallest point found."
            break
        if (Tf is not None) and (schedule.T < Tf):
            retval = 1
            break
        if (maxeval is not None) and (schedule.feval > maxeval):
            retval = 2
            break
        if (iters > maxiter):
            print "Warning: Maximum number of iterations exceeded."
            retval = 3
            break
        if (maxaccept is not None) and (schedule.accepted > maxaccept):
            retval = 4
            break

    if full_output:
        return best_state.x, best_state.cost, schedule.T, \
               schedule.feval, iters, schedule.accepted, retval
    else:
        return best_state.x, retval



if __name__ == "__main__":
    from numpy import cos
    # minimum expected at ~-0.195
    func = lambda x: cos(14.5*x-0.3) + (x+0.2)*x
    print anneal(func,1.0,full_output=1,upper=3.0,lower=-3.0,feps=1e-4,maxiter=2000,schedule='cauchy')
    print anneal(func,1.0,full_output=1,upper=3.0,lower=-3.0,feps=1e-4,maxiter=2000,schedule='fast')
    print anneal(func,1.0,full_output=1,upper=3.0,lower=-3.0,feps=1e-4,maxiter=2000,schedule='boltzmann')

    # minimum expected at ~[-0.195, -0.1]
    func = lambda x: cos(14.5*x[0]-0.3) + (x[1]+0.2)*x[1] + (x[0]+0.2)*x[0]
    print anneal(func,[1.0, 1.0],full_output=1,upper=[3.0, 3.0],lower=[-3.0, -3.0],feps=1e-4,maxiter=2000,schedule='cauchy')
    print anneal(func,[1.0, 1.0],full_output=1,upper=[3.0, 3.0],lower=[-3.0, -3.0],feps=1e-4,maxiter=2000,schedule='fast')
    print anneal(func,[1.0, 1.0],full_output=1,upper=[3.0, 3.0],lower=[-3.0, -3.0],feps=1e-4,maxiter=2000,schedule='boltzmann')



from distutils.core import setup, Extension

import os, glob, numpy, sys
if 'upload' in sys.argv:
    from ez_setup import use_setuptools; use_setuptools()
    from setuptools import setup, Extension

__version__ = open('VERSION').read().strip()

def get_description():
    lines = [L.strip() for L in open('README').readlines()]
    d_start = None
    for cnt, L in enumerate(lines):
        if L.startswith('DESCRIPTION'): d_start = cnt + 1
        elif not d_start is None:
            if len(L) == 0: return ' '.join(lines[d_start:cnt])
    raise RuntimeError('Bad README')

def indir(path, files):
    return [os.path.join(path, f) for f in files]

setup(name = 'aipy',
    version = __version__,
    description = 'Astronomical Interferometry in PYthon',
    long_description = get_description(),
    license = 'GPL',
    author = 'Aaron Parsons',
    author_email = 'aparsons@astron.berkeley.edu',
    url = 'http://setiathome.berkeley.edu/~aparsons/aipy',
    classifiers = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Topic :: Scientific/Engineering :: Astronomy',
    ],
    setup_requires = ['numpy>=1.2'],
    install_requires = ['pyephem>=3.7.3.2', 'pyfits>=1.3', 'numpy>=1.2'],
    dependency_links = [
        'http://www.stsci.edu/resources/software_hardware/pyfits'
    ],
    package_dir = {'aipy':'src', 'aipy.optimize':'src/optimize'},
    packages = ['aipy', 'aipy.optimize'],
    ext_modules = [
        Extension('aipy._healpix',
            ['src/_healpix/healpix_wrap.cpp', 
            'src/_healpix/cxx/Healpix_cxx/healpix_base.cc'],
            include_dirs = [numpy.get_include(), 'src/_healpix/cxx/cxxsupport',
                'src/_healpix/cxx/Healpix_cxx']),
        Extension('aipy._alm',
            ['src/_healpix/alm_wrap.cpp', 
            'src/_healpix/cxx/Healpix_cxx/alm_map_tools.cc',
            'src/_healpix/cxx/libfftpack/ls_fft.c',
            'src/_healpix/cxx/libfftpack/bluestein.c',
            'src/_healpix/cxx/libfftpack/fftpack.c',
            'src/_healpix/cxx/Healpix_cxx/healpix_map.cc',
            'src/_healpix/cxx/Healpix_cxx/healpix_base.cc'],
            include_dirs = [numpy.get_include(), 'src/_healpix/cxx/cxxsupport',
                'src/_healpix/cxx/Healpix_cxx']),
        Extension('aipy._miriad', ['src/_miriad/miriad_wrap.cpp'] + \
            indir('src/_miriad/mir', ['uvio.c','hio.c','pack.c','bug.c',
                'dio.c','headio.c','maskio.c']),
            include_dirs = [numpy.get_include(), 'src/_miriad', 
                'src/_miriad/mir']),
        Extension('aipy._deconv', ['src/_deconv/deconv.cpp'],
            include_dirs = [numpy.get_include()]),
        Extension('aipy.utils', ['src/utils/utils.cpp'],
            include_dirs = [numpy.get_include()]),
        Extension('aipy._cephes',
            ['src/_cephes/_cephesmodule.c', 'src/_cephes/ufunc_extras.c'] + \
            glob.glob('src/_cephes/cephes/*.c') + \
            glob.glob('src/_cephes/c_misc/*.c'),
            include_dirs = [numpy.get_include()]),
    ],
    scripts=glob.glob('scripts/*'),
    package_data = {'aipy': ['doc/*.tex', 'doc/*.png']},
)



#! /usr/bin/env python
"""
A script for fitting the delays, positions, and offsets of antennas given
starting parameters in "aipy.loc" and a list of sources (from "aipy.src").

Author: Aaron Parsons
"""

import aipy as a, numpy as n, sys, os, optparse

def calc_gain(phsdata):
    img = n.fft.ifft(phsdata.filled(0))
    return img * fit_window(img.shape[0])

def fit_window(width):
    w = n.arange(width)
    w = n.where(w > width/2, width - w, w)
    return 10.**(-w)

o = optparse.OptionParser()
o.set_usage('fit_dly_pos.py [options] *.uv')
o.set_description(__doc__)
a.scripting.add_standard_options(o, ant=True, pol=True, loc=True, 
    src=True, dec=True)
o.add_option('--fitants', dest='fitants',
    help='List of antennas to fit parameters for.')
o.add_option('--swath', dest='swath', default=0, type='int',
    help='Number of bins around center to wipe out when removing a src.')
o.add_option('--prms', dest='prms', 
    help='Comma delimited list of paramters to fit (x,y,z,delay,offset).')
opts, args = o.parse_args(sys.argv[1:])

# Parse command-line options
uv = a.miriad.UV(args[0])
a.scripting.uv_selector(uv, opts.ant, opts.pol)
aa = a.loc.get_aa(opts.loc, uv['sdf'], uv['sfreq'], uv['nchan'])
cat = a.scripting.parse_src(opts.src, force_cat=True)
fitants = map(int, opts.fitants.split(','))
prms = opts.prms.split(',')
del(uv)

# Generate list of parameters to fit
prm_dict = {}
for a in fitants: prm_dict[a] = prms
print prm_dict
start_prms = aa.get_params(prm_dict)
prm_list, key_list = a.fit.flatten_prms(start_prms)

first_fit = None    # Used to normalize fit values to the starting fit

def fit_func(prms):
    global first_fit
    prms = a.fit.reconstruct_prms(prms, key_list)
    a.fit.print_params(prms)
    aa.set_params(prms)
    score = 0
    cnt,curtime = 0,None
    src_phs = {}
    for uvfile in args:
        sys.stdout.write('.'), ; sys.stdout.flush()
        uv = a.miriad.UV(uvfile)
        a.scripting.uv_selector(uv, opts.ant, opts.pol)
        for (uvw,t,(i,j)),d,f in uv.all(raw=True):
            # Use only every Nth integration, if decimation is specified.
            if curtime != t:
                curtime = t
                cnt = (cnt + 1) % opts.decimate
                for s in src_phs:
                    score += abs(src_phs[s]).sum()
                    src_phs = {}
                if cnt == 0:
                    aa.set_jultime(t)
                    cat.compute(aa)
            if cnt != 0: continue
            for s in cat:
                ds = n.where(f, 0, d)
                # Remove all other interfering sources from data
                for rs in cat:
                    if rs != s: ds = aa.rmsrc(ds, cat[rs], i, j)
                ds = n.where(f, 0, d)
                try:
                    ds = aa.phs2src(ds, cat[s], i, j)
                    src_phs[s] = src_phs.get(s,0) + calc_gain(ds)
                except(a.ant.PointingError): pass
    score = 1 / score
    if first_fit is None: first_fit = score
    print 'Score:', score, 
    print '(%2.2f%% of %f)' % (100 * score / first_fit, first_fit)
    print '-------------------------------------------------------------------'
    return score / first_fit

a.optimize.fmin(fit_func, prm_list)
print 'Done!'




#! /usr/bin/env python
"""
Subtract the data in the second UV file from the data in the first UV file.
I imagine they better have corresponding integrations/data order.

Author: Aaron Parsons
"""

import aipy as a, numpy as n, sys, optparse

o = optparse.OptionParser()
o.set_usage('difuv.py [options] file1.uv file2.uv')
o.set_description(__doc__)
opts,args = o.parse_args(sys.argv[1:])

assert(len(args) == 2)
uv1 = a.miriad.UV(args[0])
uv2 = a.miriad.UV(args[1])

def mfunc(uv, p, d, f):
    p2,d2,f2 = uv2.read(raw=True)
    f = n.logical_or(f,f2)
    d = d - d2
    return p, d, f

uvo = a.miriad.UV(args[0] + 'd', status='new')
uvo.init_from_uv(uv1)
uvo.pipe(uv1, mfunc=mfunc, raw=True, 
    append2hist='DIFUV: subtracted file %s\n' % args[1])



# ******NOTICE***************
# optimize.py module by Travis E. Oliphant
#
# You may copy and use this module as you see fit with no
# guarantee implied provided you keep this notice in all copies.
# *****END NOTICE************

# A collection of optimization algorithms.  Version 0.5
# Restricted to pure-python: Aaron Parsons

# Minimization routines

__all__ = ['fmin', 'fmin_powell','fmin_ncg',
           'fminbound','brent', 'golden','bracket','rosen','rosen_der',
           'rosen_hess', 'rosen_hess_prod', 'brute', 'approx_fprime',
           'line_search', 'check_grad']

import numpy
from numpy import atleast_1d, eye, mgrid, argmin, zeros, shape, empty, \
     squeeze, isscalar, vectorize, asarray, absolute, sqrt, Inf, asfarray, isinf

# These have been copied from Numeric's MLab.py
# I don't think they made the transition to scipy_core
def max(m,axis=0):
    """max(m,axis=0) returns the maximum of m along dimension axis.
    """
    m = asarray(m)
    return numpy.maximum.reduce(m,axis)

def min(m,axis=0):
    """min(m,axis=0) returns the minimum of m along dimension axis.
    """
    m = asarray(m)
    return numpy.minimum.reduce(m,axis)

abs = absolute
import __builtin__
pymin = __builtin__.min
pymax = __builtin__.max
__version__="0.7"
_epsilon = sqrt(numpy.finfo(float).eps)

def vecnorm(x, ord=2):
    if ord == Inf:
        return numpy.amax(abs(x))
    elif ord == -Inf:
        return numpy.amin(abs(x))
    else:
        return numpy.sum(abs(x)**ord,axis=0)**(1.0/ord)

def rosen(x):  # The Rosenbrock function
    x = asarray(x)
    return numpy.sum(100.0*(x[1:]-x[:-1]**2.0)**2.0 + (1-x[:-1])**2.0,axis=0)

def rosen_der(x):
    x = asarray(x)
    xm = x[1:-1]
    xm_m1 = x[:-2]
    xm_p1 = x[2:]
    der = numpy.zeros_like(x)
    der[1:-1] = 200*(xm-xm_m1**2) - 400*(xm_p1 - xm**2)*xm - 2*(1-xm)
    der[0] = -400*x[0]*(x[1]-x[0]**2) - 2*(1-x[0])
    der[-1] = 200*(x[-1]-x[-2]**2)
    return der

def rosen_hess(x):
    x = atleast_1d(x)
    H = numpy.diag(-400*x[:-1],1) - numpy.diag(400*x[:-1],-1)
    diagonal = numpy.zeros(len(x), dtype=x.dtype)
    diagonal[0] = 1200*x[0]-400*x[1]+2
    diagonal[-1] = 200
    diagonal[1:-1] = 202 + 1200*x[1:-1]**2 - 400*x[2:]
    H = H + numpy.diag(diagonal)
    return H

def rosen_hess_prod(x,p):
    x = atleast_1d(x)
    Hp = numpy.zeros(len(x), dtype=x.dtype)
    Hp[0] = (1200*x[0]**2 - 400*x[1] + 2)*p[0] - 400*x[0]*p[1]
    Hp[1:-1] = -400*x[:-2]*p[:-2]+(202+1200*x[1:-1]**2-400*x[2:])*p[1:-1] \
               -400*x[1:-1]*p[2:]
    Hp[-1] = -400*x[-2]*p[-2] + 200*p[-1]
    return Hp

def wrap_function(function, args):
    ncalls = [0]
    def function_wrapper(x):
        ncalls[0] += 1
        return function(x, *args)
    return ncalls, function_wrapper

def fmin(func, x0, args=(), xtol=1e-4, ftol=1e-4, maxiter=None, maxfun=None,
         full_output=0, disp=1, retall=0, callback=None):
    """Minimize a function using the downhill simplex algorithm.
    
    :Parameters:

      func : the Python function or method to be minimized.
      x0 : ndarray - the initial guess.
      args : extra arguments for func.
      callback : an optional user-supplied function to call after each
                  iteration.  It is called as callback(xk), where xk is the
                  current parameter vector.

    :Returns: (xopt, {fopt, iter, funcalls, warnflag})

      xopt : ndarray 
        minimizer of function
      fopt : number 
        value of function at minimum: fopt = func(xopt)
      iter : number 
        number of iterations
      funcalls : number
        number of function calls
      warnflag : number 
        Integer warning flag:
                  1 : 'Maximum number of function evaluations.'
                  2 : 'Maximum number of iterations.'
      allvecs : Python list 
        a list of solutions at each iteration

    :OtherParameters:

      xtol : number 
        acceptable relative error in xopt for convergence.
      ftol : number 
        acceptable relative error in func(xopt) for convergence.
      maxiter : number 
        the maximum number of iterations to perform.
      maxfun : number 
        the maximum number of function evaluations.
      full_output : number 
        non-zero if fval and warnflag outputs are desired.
      disp : number 
        non-zero to print convergence messages.
      retall : number 
        non-zero to return list of solutions at each iteration

    :SeeAlso:

      fmin, fmin_powell, fmin_cg,
             fmin_bfgs, fmin_ncg -- multivariate local optimizers
      leastsq -- nonlinear least squares minimizer

      fmin_l_bfgs_b, fmin_tnc,
             fmin_cobyla -- constrained multivariate optimizers

      anneal, brute -- global optimizers

      fminbound, brent, golden, bracket -- local scalar minimizers

      fsolve -- n-dimenstional root-finding

      brentq, brenth, ridder, bisect, newton -- one-dimensional root-finding

      fixed_point -- scalar fixed-point finder
      
    Notes
    
    -----------
    
      Uses a Nelder-Mead simplex algorithm to find the minimum of function
      of one or more variables.
      """
    fcalls, func = wrap_function(func, args)
    x0 = asfarray(x0).flatten()
    N = len(x0)
    rank = len(x0.shape)
    if not -1 < rank < 2:
        raise ValueError, "Initial guess must be a scalar or rank-1 sequence."
    if maxiter is None:
        maxiter = N * 200
    if maxfun is None:
        maxfun = N * 200

    rho = 1; chi = 2; psi = 0.5; sigma = 0.5;
    one2np1 = range(1,N+1)

    if rank == 0:
        sim = numpy.zeros((N+1,), dtype=x0.dtype)
    else:
        sim = numpy.zeros((N+1,N), dtype=x0.dtype)
    fsim = numpy.zeros((N+1,), float)
    sim[0] = x0
    if retall:
        allvecs = [sim[0]]
    fsim[0] = func(x0)
    nonzdelt = 0.05
    zdelt = 0.00025
    for k in range(0,N):
        y = numpy.array(x0,copy=True)
        if y[k] != 0:
            y[k] = (1+nonzdelt)*y[k]
        else:
            y[k] = zdelt

        sim[k+1] = y
        f = func(y)
        fsim[k+1] = f

    ind = numpy.argsort(fsim)
    fsim = numpy.take(fsim,ind,0)
    # sort so sim[0,:] has the lowest function value
    sim = numpy.take(sim,ind,0)

    iterations = 1

    while (fcalls[0] < maxfun and iterations < maxiter):
        if (max(numpy.ravel(abs(sim[1:]-sim[0]))) <= xtol \
            and max(abs(fsim[0]-fsim[1:])) <= ftol):
            break

        xbar = numpy.add.reduce(sim[:-1],0) / N
        xr = (1+rho)*xbar - rho*sim[-1]
        fxr = func(xr)
        doshrink = 0

        if fxr < fsim[0]:
            xe = (1+rho*chi)*xbar - rho*chi*sim[-1]
            fxe = func(xe)

            if fxe < fxr:
                sim[-1] = xe
                fsim[-1] = fxe
            else:
                sim[-1] = xr
                fsim[-1] = fxr
        else: # fsim[0] <= fxr
            if fxr < fsim[-2]:
                sim[-1] = xr
                fsim[-1] = fxr
            else: # fxr >= fsim[-2]
                # Perform contraction
                if fxr < fsim[-1]:
                    xc = (1+psi*rho)*xbar - psi*rho*sim[-1]
                    fxc = func(xc)

                    if fxc <= fxr:
                        sim[-1] = xc
                        fsim[-1] = fxc
                    else:
                        doshrink=1
                else:
                    # Perform an inside contraction
                    xcc = (1-psi)*xbar + psi*sim[-1]
                    fxcc = func(xcc)

                    if fxcc < fsim[-1]:
                        sim[-1] = xcc
                        fsim[-1] = fxcc
                    else:
                        doshrink = 1

                if doshrink:
                    for j in one2np1:
                        sim[j] = sim[0] + sigma*(sim[j] - sim[0])
                        fsim[j] = func(sim[j])

        ind = numpy.argsort(fsim)
        sim = numpy.take(sim,ind,0)
        fsim = numpy.take(fsim,ind,0)
        if callback is not None:
            callback(sim[0])
        iterations += 1
        if retall:
            allvecs.append(sim[0])

    x = sim[0]
    fval = min(fsim)
    warnflag = 0

    if fcalls[0] >= maxfun:
        warnflag = 1
        if disp:
            print "Warning: Maximum number of function evaluations has "\
                  "been exceeded."
    elif iterations >= maxiter:
        warnflag = 2
        if disp:
            print "Warning: Maximum number of iterations has been exceeded"
    else:
        if disp:
            print "Optimization terminated successfully."
            print "         Current function value: %f" % fval
            print "         Iterations: %d" % iterations
            print "         Function evaluations: %d" % fcalls[0]


    if full_output:
        retlist = x, fval, iterations, fcalls[0], warnflag
        if retall:
            retlist += (allvecs,)
    else:
        retlist = x
        if retall:
            retlist = (x, allvecs)

    return retlist



def _cubicmin(a,fa,fpa,b,fb,c,fc):
    # finds the minimizer for a cubic polynomial that goes through the
    #  points (a,fa), (b,fb), and (c,fc) with derivative at a of fpa.
    #
    # if no minimizer can be found return None
    #
    # f(x) = A *(x-a)^3 + B*(x-a)^2 + C*(x-a) + D

    C = fpa
    D = fa
    db = b-a
    dc = c-a
    if (db == 0) or (dc == 0) or (b==c): return None
    denom = (db*dc)**2 * (db-dc)
    d1 = empty((2,2))
    d1[0,0] = dc**2
    d1[0,1] = -db**2
    d1[1,0] = -dc**3
    d1[1,1] = db**3
    [A,B] = numpy.dot(d1,asarray([fb-fa-C*db,fc-fa-C*dc]).flatten())
    A /= denom
    B /= denom
    radical = B*B-3*A*C
    if radical < 0:  return None
    if (A == 0): return None
    xmin = a + (-B + sqrt(radical))/(3*A)
    return xmin


def _quadmin(a,fa,fpa,b,fb):
    # finds the minimizer for a quadratic polynomial that goes through
    #  the points (a,fa), (b,fb) with derivative at a of fpa
    # f(x) = B*(x-a)^2 + C*(x-a) + D
    D = fa
    C = fpa
    db = b-a*1.0
    if (db==0): return None
    B = (fb-D-C*db)/(db*db)
    if (B <= 0): return None
    xmin = a  - C / (2.0*B)
    return xmin

def zoom(a_lo, a_hi, phi_lo, phi_hi, derphi_lo,
         phi, derphi, phi0, derphi0, c1, c2):
    maxiter = 10
    i = 0
    delta1 = 0.2  # cubic interpolant check
    delta2 = 0.1  # quadratic interpolant check
    phi_rec = phi0
    a_rec = 0
    while 1:
        # interpolate to find a trial step length between a_lo and a_hi
        # Need to choose interpolation here.  Use cubic interpolation and then if the
        #  result is within delta * dalpha or outside of the interval bounded by a_lo or a_hi
        #  then use quadratic interpolation, if the result is still too close, then use bisection

        dalpha = a_hi-a_lo;
        if dalpha < 0: a,b = a_hi,a_lo
        else: a,b = a_lo, a_hi

        # minimizer of cubic interpolant
        #    (uses phi_lo, derphi_lo, phi_hi, and the most recent value of phi)
        #      if the result is too close to the end points (or out of the interval)
        #         then use quadratic interpolation with phi_lo, derphi_lo and phi_hi
        #      if the result is stil too close to the end points (or out of the interval)
        #         then use bisection

        if (i > 0):
            cchk = delta1*dalpha
            a_j = _cubicmin(a_lo, phi_lo, derphi_lo, a_hi, phi_hi, a_rec, phi_rec)
        if (i==0) or (a_j is None) or (a_j > b-cchk) or (a_j < a+cchk):
            qchk = delta2*dalpha
            a_j = _quadmin(a_lo, phi_lo, derphi_lo, a_hi, phi_hi)
            if (a_j is None) or (a_j > b-qchk) or (a_j < a+qchk):
                a_j = a_lo + 0.5*dalpha
#                print "Using bisection."
#            else: print "Using quadratic."
#        else: print "Using cubic."

        # Check new value of a_j

        phi_aj = phi(a_j)
        if (phi_aj > phi0 + c1*a_j*derphi0) or (phi_aj >= phi_lo):
            phi_rec = phi_hi
            a_rec = a_hi
            a_hi = a_j
            phi_hi = phi_aj
        else:
            derphi_aj = derphi(a_j)
            if abs(derphi_aj) <= -c2*derphi0:
                a_star = a_j
                val_star = phi_aj
                valprime_star = derphi_aj
                break
            if derphi_aj*(a_hi - a_lo) >= 0:
                phi_rec = phi_hi
                a_rec = a_hi
                a_hi = a_lo
                phi_hi = phi_lo
            else:
                phi_rec = phi_lo
                a_rec = a_lo
            a_lo = a_j
            phi_lo = phi_aj
            derphi_lo = derphi_aj
        i += 1
        if (i > maxiter):
            a_star = a_j
            val_star = phi_aj
            valprime_star = None
            break
    return a_star, val_star, valprime_star

def line_search(f, myfprime, xk, pk, gfk, old_fval, old_old_fval,
                args=(), c1=1e-4, c2=0.9, amax=50):
    """Find alpha that satisfies strong Wolfe conditions.
    
    :Parameters:
    
    f : objective function
    myfprime : objective function gradient (can be None)
    xk : ndarray -- start point 
    pk : ndarray -- search direction
    gfk : ndarray -- gradient value for x=xk 
    args : additional arguments for user functions
    c1 : number -- parameter for Armijo condition rule
    c2 : number - parameter for curvature condition rule
    
    :Returns: 
    
    alpha0 : number -- required alpha (x_new = x0 + alpha * pk)
    fc : number of function evaluations 
    gc : number of gradient evaluations 
    
    
    Notes
    
    --------------------------------
    
    Uses the line search algorithm to enforce strong Wolfe conditions
    Wright and Nocedal, 'Numerical Optimization', 1999, pg. 59-60

    For the zoom phase it uses an algorithm by

    """

    global _ls_fc, _ls_gc, _ls_ingfk
    _ls_fc = 0
    _ls_gc = 0
    _ls_ingfk = None
    def phi(alpha):
        global _ls_fc
        _ls_fc += 1
        return f(xk+alpha*pk,*args)

    if isinstance(myfprime,type(())):
        def phiprime(alpha):
            global _ls_fc, _ls_ingfk
            _ls_fc += len(xk)+1
            eps = myfprime[1]
            fprime = myfprime[0]
            newargs = (f,eps) + args
            _ls_ingfk = fprime(xk+alpha*pk,*newargs)  # store for later use
            return numpy.dot(_ls_ingfk,pk)
    else:
        fprime = myfprime
        def phiprime(alpha):
            global _ls_gc, _ls_ingfk
            _ls_gc += 1
            _ls_ingfk = fprime(xk+alpha*pk,*args)  # store for later use
            return numpy.dot(_ls_ingfk,pk)

    alpha0 = 0
    phi0 = old_fval
    derphi0 = numpy.dot(gfk,pk)

    alpha1 = pymin(1.0,1.01*2*(phi0-old_old_fval)/derphi0)

    if alpha1 == 0:
        # This shouldn't happen. Perhaps the increment has slipped below
        # machine precision?  For now, set the return variables skip the
        # useless while loop, and raise warnflag=2 due to possible imprecision.
        alpha_star = None
        fval_star = old_fval
        old_fval = old_old_fval
        fprime_star = None

    phi_a1 = phi(alpha1)
    #derphi_a1 = phiprime(alpha1)  evaluated below

    phi_a0 = phi0
    derphi_a0 = derphi0

    i = 1
    maxiter = 10
    while 1:         # bracketing phase
        if alpha1 == 0:
            break
        if (phi_a1 > phi0 + c1*alpha1*derphi0) or \
           ((phi_a1 >= phi_a0) and (i > 1)):
            alpha_star, fval_star, fprime_star = \
                        zoom(alpha0, alpha1, phi_a0,
                             phi_a1, derphi_a0, phi, phiprime,
                             phi0, derphi0, c1, c2)
            break

        derphi_a1 = phiprime(alpha1)
        if (abs(derphi_a1) <= -c2*derphi0):
            alpha_star = alpha1
            fval_star = phi_a1
            fprime_star = derphi_a1
            break

        if (derphi_a1 >= 0):
            alpha_star, fval_star, fprime_star = \
                        zoom(alpha1, alpha0, phi_a1,
                             phi_a0, derphi_a1, phi, phiprime,
                             phi0, derphi0, c1, c2)
            break

        alpha2 = 2 * alpha1   # increase by factor of two on each iteration
        i = i + 1
        alpha0 = alpha1
        alpha1 = alpha2
        phi_a0 = phi_a1
        phi_a1 = phi(alpha1)
        derphi_a0 = derphi_a1

        # stopping test if lower function not found
        if (i > maxiter):
            alpha_star = alpha1
            fval_star = phi_a1
            fprime_star = None
            break

    if fprime_star is not None:
        # fprime_star is a number (derphi) -- so use the most recently calculated gradient
        #                                     used in computing it derphi = gfk*pk
        #                                     this is the gradient at the next step
        #                                     no need to compute it again in the outer loop.
        fprime_star = _ls_ingfk

    return alpha_star, _ls_fc, _ls_gc, fval_star, old_fval, fprime_star


def line_search_BFGS(f, xk, pk, gfk, old_fval, args=(), c1=1e-4, alpha0=1):
    """Minimize over alpha, the function f(xk+alpha pk)

    Uses the interpolation algorithm (Armiijo backtracking) as suggested by
    Wright and Nocedal in 'Numerical Optimization', 1999, pg. 56-57

    :Returns: (alpha, fc, gc)
    """
    
    xk = atleast_1d(xk)
    fc = 0
    phi0 = old_fval                            # compute f(xk) -- done in past loop
    phi_a0 = f(*((xk+alpha0*pk,)+args))     
    fc = fc + 1
    derphi0 = numpy.dot(gfk,pk)

    if (phi_a0 <= phi0 + c1*alpha0*derphi0):
        return alpha0, fc, 0, phi_a0

    # Otherwise compute the minimizer of a quadratic interpolant:

    alpha1 = -(derphi0) * alpha0**2 / 2.0 / (phi_a0 - phi0 - derphi0 * alpha0)
    phi_a1 = f(*((xk+alpha1*pk,)+args))
    fc = fc + 1

    if (phi_a1 <= phi0 + c1*alpha1*derphi0):
        return alpha1, fc, 0, phi_a1

    # Otherwise loop with cubic interpolation until we find an alpha which
    # satifies the first Wolfe condition (since we are backtracking, we will
    # assume that the value of alpha is not too small and satisfies the second
    # condition.

    while 1:       # we are assuming pk is a descent direction
        factor = alpha0**2 * alpha1**2 * (alpha1-alpha0)
        a = alpha0**2 * (phi_a1 - phi0 - derphi0*alpha1) - \
            alpha1**2 * (phi_a0 - phi0 - derphi0*alpha0)
        a = a / factor
        b = -alpha0**3 * (phi_a1 - phi0 - derphi0*alpha1) + \
            alpha1**3 * (phi_a0 - phi0 - derphi0*alpha0)
        b = b / factor

        alpha2 = (-b + numpy.sqrt(abs(b**2 - 3 * a * derphi0))) / (3.0*a)
        phi_a2 = f(*((xk+alpha2*pk,)+args))
        fc = fc + 1

        if (phi_a2 <= phi0 + c1*alpha2*derphi0):
            return alpha2, fc, 0, phi_a2

        if (alpha1 - alpha2) > alpha1 / 2.0 or (1 - alpha2/alpha1) < 0.96:
            alpha2 = alpha1 / 2.0

        alpha0 = alpha1
        alpha1 = alpha2
        phi_a0 = phi_a1
        phi_a1 = phi_a2


def approx_fprime(xk,f,epsilon,*args):
    f0 = f(*((xk,)+args))
    grad = numpy.zeros((len(xk),), float)
    ei = numpy.zeros((len(xk),), float)
    for k in range(len(xk)):
        ei[k] = epsilon
        grad[k] = (f(*((xk+ei,)+args)) - f0)/epsilon
        ei[k] = 0.0
    return grad

def check_grad(func, grad, x0, *args):
    return sqrt(sum((grad(x0,*args)-approx_fprime(x0,func,_epsilon,*args))**2))

def approx_fhess_p(x0,p,fprime,epsilon,*args):
    f2 = fprime(*((x0+epsilon*p,)+args))
    f1 = fprime(*((x0,)+args))
    return (f2 - f1)/epsilon


def fmin_ncg(f, x0, fprime, fhess_p=None, fhess=None, args=(), avextol=1e-5,
             epsilon=_epsilon, maxiter=None, full_output=0, disp=1, retall=0,
             callback=None):
    """ Minimize the function f using the Newton-CG method. 

    :Parameters:

    f -- the Python function or method to be minimized.
    x0 : ndarray -- the initial guess for the minimizer.
    fprime -- a function to compute the gradient of f: fprime(x, *args)
    fhess_p -- a function to compute the Hessian of f times an
               arbitrary vector: fhess_p (x, p, *args)
    fhess -- a function to compute the Hessian matrix of f.
    args -- extra arguments for f, fprime, fhess_p, and fhess (the same
            set of extra arguments is supplied to all of these functions).

    epsilon : number 
        if fhess is approximated use this value for
                 the step size (can be scalar or vector)
    callback -- an optional user-supplied function to call after each
                iteration.  It is called as callback(xk), where xk is the
                current parameter vector.

    :Returns: (xopt, {fopt, fcalls, gcalls, hcalls, warnflag},{allvecs})

    xopt : ndarray
        the minimizer of f
    fopt : number
        the value of the function at xopt: fopt = f(xopt)
    fcalls : number
        the number of function calls
    gcalls : number
        the number of gradient calls
    hcalls : number 
        the number of hessian calls.
    warnflag : number
        algorithm warnings:
                1 : 'Maximum number of iterations exceeded.'
    allvecs : Python list
        a list of all tried iterates

    :OtherParameters:

    avextol : number
        Convergence is assumed when the average relative error in
               the minimizer falls below this amount.
    maxiter : number
        Maximum number of iterations to allow.
    full_output : number
        If non-zero return the optional outputs.
    disp : number
        If non-zero print convergence message.
    retall : bool
        return a list of results at each iteration if True

    :SeeAlso:

      fmin, fmin_powell, fmin_cg,
             fmin_bfgs, fmin_ncg -- multivariate local optimizers
      leastsq -- nonlinear least squares minimizer

      fmin_l_bfgs_b, fmin_tnc,
             fmin_cobyla -- constrained multivariate optimizers

      anneal, brute -- global optimizers

      fminbound, brent, golden, bracket -- local scalar minimizers

      fsolve -- n-dimenstional root-finding

      brentq, brenth, ridder, bisect, newton -- one-dimensional root-finding

      fixed_point -- scalar fixed-point finder

    Notes
    
    ---------------------------------------------
    
    Only one of fhess_p or fhess need be given.  If fhess is provided,
    then fhess_p will be ignored.  If neither fhess nor fhess_p is
    provided, then the hessian product will be approximated using finite
    differences on fprime. fhess_p must compute the hessian times an arbitrary
    vector. If it is not given, finite-differences on fprime are used to
    compute it. See Wright, and Nocedal 'Numerical Optimization', 1999,
    pg. 140.

    """
    x0 = asarray(x0).flatten()
    fcalls, f = wrap_function(f, args)
    gcalls, fprime = wrap_function(fprime, args)
    hcalls = 0
    if maxiter is None:
        maxiter = len(x0)*200

    xtol = len(x0)*avextol
    update = [2*xtol]
    xk = x0
    if retall:
        allvecs = [xk]
    k = 0
    old_fval = f(x0)
    while (numpy.add.reduce(abs(update)) > xtol) and (k < maxiter):
        # Compute a search direction pk by applying the CG method to
        #  del2 f(xk) p = - grad f(xk) starting from 0.
        b = -fprime(xk)
        maggrad = numpy.add.reduce(abs(b))
        eta = min([0.5,numpy.sqrt(maggrad)])
        termcond = eta * maggrad
        xsupi = zeros(len(x0), dtype=x0.dtype)
        ri = -b
        psupi = -ri
        i = 0
        dri0 = numpy.dot(ri,ri)

        if fhess is not None:             # you want to compute hessian once.
            A = fhess(*(xk,)+args)
            hcalls = hcalls + 1

        while numpy.add.reduce(abs(ri)) > termcond:
            if fhess is None:
                if fhess_p is None:
                    Ap = approx_fhess_p(xk,psupi,fprime,epsilon)
                else:
                    Ap = fhess_p(xk,psupi, *args)
                    hcalls = hcalls + 1
            else:
                Ap = numpy.dot(A,psupi)
            # check curvature
            Ap = asarray(Ap).squeeze() # get rid of matrices...
            curv = numpy.dot(psupi,Ap)
            if curv == 0.0:
                break
            elif curv < 0:
                if (i > 0):
                    break
                else:
                    xsupi = xsupi + dri0/curv * psupi
                    break
            alphai = dri0 / curv
            xsupi = xsupi + alphai * psupi
            ri = ri + alphai * Ap
            dri1 = numpy.dot(ri,ri)
            betai = dri1 / dri0
            psupi = -ri + betai * psupi
            i = i + 1
            dri0 = dri1          # update numpy.dot(ri,ri) for next time.

        pk = xsupi  # search direction is solution to system.
        gfk = -b    # gradient at xk
        alphak, fc, gc, old_fval = line_search_BFGS(f,xk,pk,gfk,old_fval)

        update = alphak * pk
        xk = xk + update        # upcast if necessary
        if callback is not None:
            callback(xk)
        if retall:
            allvecs.append(xk)
        k += 1

    if disp or full_output:
        fval = old_fval
    if k >= maxiter:
        warnflag = 1
        if disp:
            print "Warning: Maximum number of iterations has been exceeded"
            print "         Current function value: %f" % fval
            print "         Iterations: %d" % k
            print "         Function evaluations: %d" % fcalls[0]
            print "         Gradient evaluations: %d" % gcalls[0]
            print "         Hessian evaluations: %d" % hcalls
    else:
        warnflag = 0
        if disp:
            print "Optimization terminated successfully."
            print "         Current function value: %f" % fval
            print "         Iterations: %d" % k
            print "         Function evaluations: %d" % fcalls[0]
            print "         Gradient evaluations: %d" % gcalls[0]
            print "         Hessian evaluations: %d" % hcalls

    if full_output:
        retlist = xk, fval, fcalls[0], gcalls[0], hcalls, warnflag
        if retall:
            retlist += (allvecs,)
    else:
        retlist = xk
        if retall:
            retlist = (xk, allvecs)

    return retlist


def fminbound(func, x1, x2, args=(), xtol=1e-5, maxfun=500,
              full_output=0, disp=1):
    """Bounded minimization for scalar functions.

    :Parameters:

      func -- the function to be minimized (must accept scalar input and return
              scalar output).
      x1, x2 : ndarray 
        the optimization bounds.
      args -- extra arguments to pass to function.
      xtol : number
        the convergence tolerance.
      maxfun : number
        maximum function evaluations.
      full_output : number
        Non-zero to return optional outputs.
      disp : number
        Non-zero to print messages.
              0 : no message printing.
              1 : non-convergence notification messages only.
              2 : print a message on convergence too.
              3 : print iteration results.


    :Returns: (xopt, {fval, ierr, numfunc})

      xopt : ndarray
        The minimizer of the function over the interval.
      fval : number
        The function value at the minimum point.
      ierr : number
        An error flag (0 if converged, 1 if maximum number of
              function calls reached).
      numfunc : number
        The number of function calls.

    :SeeAlso:

      fmin, fmin_powell, fmin_cg,
             fmin_bfgs, fmin_ncg -- multivariate local optimizers
      leastsq -- nonlinear least squares minimizer

      fmin_l_bfgs_b, fmin_tnc,
             fmin_cobyla -- constrained multivariate optimizers

      anneal, brute -- global optimizers

      fminbound, brent, golden, bracket -- local scalar minimizers

      fsolve -- n-dimenstional root-finding

      brentq, brenth, ridder, bisect, newton -- one-dimensional root-finding

      fixed_point -- scalar fixed-point finder

    Notes
    
    -------------------------------------------------------
    
    Finds a local minimizer of the scalar function func in the interval
      x1 < xopt < x2 using Brent's method.  (See brent for auto-bracketing).


    """

    if x1 > x2:
        raise ValueError, "The lower bound exceeds the upper bound."

    flag = 0
    header = ' Func-count     x          f(x)          Procedure'
    step='       initial'

    sqrt_eps = sqrt(2.2e-16)
    golden_mean = 0.5*(3.0-sqrt(5.0))
    a, b = x1, x2
    fulc = a + golden_mean*(b-a)
    nfc, xf = fulc, fulc
    rat = e = 0.0
    x = xf
    fx = func(x,*args)
    num = 1
    fmin_data = (1, xf, fx)

    ffulc = fnfc = fx
    xm = 0.5*(a+b)
    tol1 = sqrt_eps*abs(xf) + xtol / 3.0
    tol2 = 2.0*tol1

    if disp > 2:
        print (" ")
        print (header)
        print "%5.0f   %12.6g %12.6g %s" % (fmin_data + (step,))


    while ( abs(xf-xm) > (tol2 - 0.5*(b-a)) ):
        golden = 1
        # Check for parabolic fit
        if abs(e) > tol1:
            golden = 0
            r = (xf-nfc)*(fx-ffulc)
            q = (xf-fulc)*(fx-fnfc)
            p = (xf-fulc)*q - (xf-nfc)*r
            q = 2.0*(q-r)
            if q > 0.0: p = -p
            q = abs(q)
            r = e
            e = rat

            # Check for acceptability of parabola
            if ( (abs(p) < abs(0.5*q*r)) and (p > q*(a-xf)) and (p < q*(b-xf))):
                rat = (p+0.0) / q;
                x = xf + rat
                step = '       parabolic'

                if ((x-a) < tol2) or ((b-x) < tol2):
                    si = numpy.sign(xm-xf) + ((xm-xf)==0)
                    rat = tol1*si
            else:      # do a golden section step
                golden = 1

        if golden:  # Do a golden-section step
            if xf >= xm:
                e=a-xf
            else:
                e=b-xf
            rat = golden_mean*e
            step = '       golden'

        si = numpy.sign(rat) + (rat == 0)
        x = xf + si*max([abs(rat), tol1])
        fu = func(x,*args)
        num += 1
        fmin_data = (num, x, fu)
        if disp > 2:
            print "%5.0f   %12.6g %12.6g %s" % (fmin_data + (step,))

        if fu <= fx:
            if x >= xf:
                a = xf
            else:
                b = xf
            fulc, ffulc = nfc, fnfc
            nfc, fnfc = xf, fx
            xf, fx = x, fu
        else:
            if x < xf:
                a = x
            else:
                b = x
            if (fu <= fnfc) or (nfc == xf):
                fulc, ffulc = nfc, fnfc
                nfc, fnfc = x, fu
            elif (fu <= ffulc) or (fulc == xf) or (fulc == nfc):
                fulc, ffulc = x, fu

        xm = 0.5*(a+b)
        tol1 = sqrt_eps*abs(xf) + xtol/3.0
        tol2 = 2.0*tol1

        if num >= maxfun:
            flag = 1
            fval = fx
            if disp > 0:
                _endprint(x, flag, fval, maxfun, xtol, disp)
            if full_output:
                return xf, fval, flag, num
            else:
                return xf

    fval = fx
    if disp > 0:
        _endprint(x, flag, fval, maxfun, xtol, disp)

    if full_output:
        return xf, fval, flag, num
    else:
        return xf

class Brent:
    #need to rethink design of __init__
    def __init__(self, func, args=(), tol=1.48e-8, maxiter=500, full_output=0):
        self.func = func
        self.args = args
        self.tol = tol
        self.maxiter = maxiter
        self._mintol = 1.0e-11
        self._cg = 0.3819660
        self.xmin = None
        self.fval = None
        self.iter = 0
        self.funcalls = 0
        
    #need to rethink design of set_bracket (new options, etc)
    def set_bracket(self, brack = None):
        self.brack = brack
    def get_bracket_info(self):
        #set up
        func = self.func
        args = self.args
        brack = self.brack
        ### BEGIN core bracket_info code ###
        ### carefully DOCUMENT any CHANGES in core ##
        if brack is None:
            xa,xb,xc,fa,fb,fc,funcalls = bracket(func, args=args)
        elif len(brack) == 2:
            xa,xb,xc,fa,fb,fc,funcalls = bracket(func, xa=brack[0], xb=brack[1], args=args)
        elif len(brack) == 3:
            xa,xb,xc = brack
            if (xa > xc):  # swap so xa < xc can be assumed
                dum = xa; xa=xc; xc=dum
            assert ((xa < xb) and (xb < xc)), "Not a bracketing interval."
            fa = func(*((xa,)+args))
            fb = func(*((xb,)+args))
            fc = func(*((xc,)+args))
            assert ((fb<fa) and (fb < fc)), "Not a bracketing interval."
            funcalls = 3
        else:
            raise ValueError, "Bracketing interval must be length 2 or 3 sequence."
        ### END core bracket_info code ###

        return xa,xb,xc,fa,fb,fc,funcalls

    def optimize(self):
        #set up for optimization
        func = self.func
        xa,xb,xc,fa,fb,fc,funcalls = self.get_bracket_info()
        _mintol = self._mintol
        _cg = self._cg
        #################################
        #BEGIN CORE ALGORITHM
        #we are making NO CHANGES in this
        #################################
        x=w=v=xb
        fw=fv=fx=func(*((x,)+self.args))
        if (xa < xc):
            a = xa; b = xc
        else:
            a = xc; b = xa
        deltax= 0.0
        funcalls = 1
        iter = 0
        while (iter < self.maxiter):
            tol1 = self.tol*abs(x) + _mintol
            tol2 = 2.0*tol1
            xmid = 0.5*(a+b)
            if abs(x-xmid) < (tol2-0.5*(b-a)):  # check for convergence
                xmin=x; fval=fx
                break
            if (abs(deltax) <= tol1):
                if (x>=xmid): deltax=a-x       # do a golden section step
                else: deltax=b-x
                rat = _cg*deltax
            else:                              # do a parabolic step
                tmp1 = (x-w)*(fx-fv)
                tmp2 = (x-v)*(fx-fw)
                p = (x-v)*tmp2 - (x-w)*tmp1;
                tmp2 = 2.0*(tmp2-tmp1)
                if (tmp2 > 0.0): p = -p
                tmp2 = abs(tmp2)
                dx_temp = deltax
                deltax= rat
                # check parabolic fit
                if ((p > tmp2*(a-x)) and (p < tmp2*(b-x)) and (abs(p) < abs(0.5*tmp2*dx_temp))):
                    rat = p*1.0/tmp2        # if parabolic step is useful.
                    u = x + rat
                    if ((u-a) < tol2 or (b-u) < tol2):
                        if xmid-x >= 0: rat = tol1
                        else: rat = -tol1
                else:
                    if (x>=xmid): deltax=a-x # if it's not do a golden section step
                    else: deltax=b-x
                    rat = _cg*deltax

            if (abs(rat) < tol1):            # update by at least tol1
                if rat >= 0: u = x + tol1
                else: u = x - tol1
            else:
                u = x + rat
            fu = func(*((u,)+self.args))      # calculate new output value
            funcalls += 1

            if (fu > fx):                 # if it's bigger than current
                if (u<x): a=u
                else: b=u
                if (fu<=fw) or (w==x):
                    v=w; w=u; fv=fw; fw=fu
                elif (fu<=fv) or (v==x) or (v==w):
                    v=u; fv=fu
            else:
                if (u >= x): a = x
                else: b = x
                v=w; w=x; x=u
                fv=fw; fw=fx; fx=fu

            iter += 1
        #################################
        #END CORE ALGORITHM
        #################################

        self.xmin = x
        self.fval = fx
        self.iter = iter
        self.funcalls = funcalls

    def get_result(self, full_output=False):
        if full_output:
            return self.xmin, self.fval, self.iter, self.funcalls
        else:
            return self.xmin


def brent(func, args=(), brack=None, tol=1.48e-8, full_output=0, maxiter=500):
    """ Given a function of one-variable and a possible bracketing interval,
    return the minimum of the function isolated to a fractional precision of
    tol. 
    
    :Parameters:
    
    func - objective func
    args - additional arguments (if present)
    brack - triple (a,b,c) where (a<b<c) and
        func(b) < func(a),func(c).  If bracket is two numbers (a,c) then they are
        assumed to be a starting interval for a downhill bracket search
        (see bracket); it doesn't always mean that obtained solution will satisfy a<=x<=c.
        
    full_output : number
        0 - return only x (default)
        1 - return all output args (xmin, fval, iter, funcalls)
        
    :Returns:
    
    xmin : ndarray
        optim point
    fval : number
        optim value 
    iter : number
        number of iterations
    funcalls : number
        number of objective function evaluations
        
    :SeeAlso:

      fmin, fmin_powell, fmin_cg,
             fmin_bfgs, fmin_ncg -- multivariate local optimizers
      leastsq -- nonlinear least squares minimizer

      fmin_l_bfgs_b, fmin_tnc,
             fmin_cobyla -- constrained multivariate optimizers

      anneal, brute -- global optimizers

      fminbound, brent, golden, bracket -- local scalar minimizers

      fsolve -- n-dimenstional root-finding

      brentq, brenth, ridder, bisect, newton -- one-dimensional root-finding

      fixed_point -- scalar fixed-point finder

    Notes
    
    ----------------------------

    Uses inverse parabolic interpolation when possible to speed up convergence
    of golden section method.

    """

    brent = Brent(func=func, args=args, tol=tol, full_output = full_output, maxiter=maxiter)
    brent.set_bracket(brack)
    brent.optimize()
    return brent.get_result(full_output=full_output)



def golden(func, args=(), brack=None, tol=_epsilon, full_output=0):
    """ Given a function of one-variable and a possible bracketing interval,
    return the minimum of the function isolated to a fractional precision of
    tol. 

    :Parameters:
    
    func - objective function
    args - additional arguments (if present)
    brack : a triple (a,b,c) where (a<b<c) and
        func(b) < func(a),func(c).  If bracket is two numbers (a, c) then they are
        assumed to be a starting interval for a downhill bracket search
        (see bracket); it doesn't always mean that obtained solution will satisfy a<=x<=c
    tol : number 
        x tolerance stop criterion
    full_output : number
        0 for false
        1 for true
        
    :SeeAlso:

      fmin, fmin_powell, fmin_cg,
             fmin_bfgs, fmin_ncg -- multivariate local optimizers
      leastsq -- nonlinear least squares minimizer

      fmin_l_bfgs_b, fmin_tnc,
             fmin_cobyla -- constrained multivariate optimizers

      anneal, brute -- global optimizers

      fminbound, brent, golden, bracket -- local scalar minimizers

      fsolve -- n-dimenstional root-finding

      brentq, brenth, ridder, bisect, newton -- one-dimensional root-finding

      fixed_point -- scalar fixed-point finder

    Notes
    
    ---------------------------------------
    
    Uses analog of bisection method to decrease the bracketed interval.
    """
    if brack is None:
        xa,xb,xc,fa,fb,fc,funcalls = bracket(func, args=args)
    elif len(brack) == 2:
        xa,xb,xc,fa,fb,fc,funcalls = bracket(func, xa=brack[0], xb=brack[1], args=args)
    elif len(brack) == 3:
        xa,xb,xc = brack
        if (xa > xc):  # swap so xa < xc can be assumed
            dum = xa; xa=xc; xc=dum
        assert ((xa < xb) and (xb < xc)), "Not a bracketing interval."
        fa = func(*((xa,)+args))
        fb = func(*((xb,)+args))
        fc = func(*((xc,)+args))
        assert ((fb<fa) and (fb < fc)), "Not a bracketing interval."
        funcalls = 3
    else:
        raise ValueError, "Bracketing interval must be length 2 or 3 sequence."

    _gR = 0.61803399
    _gC = 1.0-_gR
    x3 = xc
    x0 = xa
    if (abs(xc-xb) > abs(xb-xa)):
        x1 = xb
        x2 = xb + _gC*(xc-xb)
    else:
        x2 = xb
        x1 = xb - _gC*(xb-xa)
    f1 = func(*((x1,)+args))
    f2 = func(*((x2,)+args))
    funcalls += 2
    while (abs(x3-x0) > tol*(abs(x1)+abs(x2))):
        if (f2 < f1):
            x0 = x1; x1 = x2; x2 = _gR*x1 + _gC*x3
            f1 = f2; f2 = func(*((x2,)+args))
        else:
            x3 = x2; x2 = x1; x1 = _gR*x2 + _gC*x0
            f2 = f1; f1 = func(*((x1,)+args))
        funcalls += 1
    if (f1 < f2):
        xmin = x1
        fval = f1
    else:
        xmin = x2
        fval = f2
    if full_output:
        return xmin, fval, funcalls
    else:
        return xmin


def bracket(func, xa=0.0, xb=1.0, args=(), grow_limit=110.0, maxiter=1000):
    """Given a function and distinct initial points, search in the downhill
    direction (as defined by the initital points) and return new points
    xa, xb, xc that bracket the minimum of the function:
    f(xa) > f(xb) < f(xc). It doesn't always mean that obtained solution will satisfy xa<=x<=xb
    
    :Parameters:
    
    func -- objective func
    xa, xb : number
        bracketing interval
    args -- additional arguments (if present)
    grow_limit : number
        max grow limit
    maxiter : number
        max iterations number
    
    :Returns: xa, xb, xc, fa, fb, fc, funcalls
    
    xa, xb, xc : number
        bracket
    fa, fb, fc : number
        objective function values in bracket
    funcalls : number
        number of function evaluations
    """
    _gold = 1.618034
    _verysmall_num = 1e-21
    fa = func(*(xa,)+args)
    fb = func(*(xb,)+args)
    if (fa < fb):                      # Switch so fa > fb
        dum = xa; xa = xb; xb = dum
        dum = fa; fa = fb; fb = dum
    xc = xb + _gold*(xb-xa)
    fc = func(*((xc,)+args))
    funcalls = 3
    iter = 0
    while (fc < fb):
        tmp1 = (xb - xa)*(fb-fc)
        tmp2 = (xb - xc)*(fb-fa)
        val = tmp2-tmp1
        if abs(val) < _verysmall_num:
            denom = 2.0*_verysmall_num
        else:
            denom = 2.0*val
        w = xb - ((xb-xc)*tmp2-(xb-xa)*tmp1)/denom
        wlim = xb + grow_limit*(xc-xb)
        if iter > maxiter:
            raise RuntimeError, "Too many iterations."
        iter += 1
        if (w-xc)*(xb-w) > 0.0:
            fw = func(*((w,)+args))
            funcalls += 1
            if (fw < fc):
                xa = xb; xb=w; fa=fb; fb=fw
                return xa, xb, xc, fa, fb, fc, funcalls
            elif (fw > fb):
                xc = w; fc=fw
                return xa, xb, xc, fa, fb, fc, funcalls
            w = xc + _gold*(xc-xb)
            fw = func(*((w,)+args))
            funcalls += 1
        elif (w-wlim)*(wlim-xc) >= 0.0:
            w = wlim
            fw = func(*((w,)+args))
            funcalls += 1
        elif (w-wlim)*(xc-w) > 0.0:
            fw = func(*((w,)+args))
            funcalls += 1
            if (fw < fc):
                xb=xc; xc=w; w=xc+_gold*(xc-xb)
                fb=fc; fc=fw; fw=func(*((w,)+args))
                funcalls += 1
        else:
            w = xc + _gold*(xc-xb)
            fw = func(*((w,)+args))
            funcalls += 1
        xa=xb; xb=xc; xc=w
        fa=fb; fb=fc; fc=fw
    return xa, xb, xc, fa, fb, fc, funcalls



def _linesearch_powell(func, p, xi, tol=1e-3):
    # line-search algorithm using fminbound
    #  find the minimium of the function
    #  func(x0+ alpha*direc)
    def myfunc(alpha):
        return func(p + alpha * xi)
    alpha_min, fret, iter, num = brent(myfunc, full_output=1, tol=tol)
    xi = alpha_min*xi
    return squeeze(fret), p+xi, xi


def fmin_powell(func, x0, args=(), xtol=1e-4, ftol=1e-4, maxiter=None,
                maxfun=None, full_output=0, disp=1, retall=0, callback=None,
                direc=None):
    """Minimize a function using modified Powell's method.

    :Parameters:

      func -- the Python function or method to be minimized.
      x0 : ndarray
        the initial guess.
      args -- extra arguments for func
      callback -- an optional user-supplied function to call after each
                  iteration.  It is called as callback(xk), where xk is the
                  current parameter vector
      direc -- initial direction set

    :Returns: (xopt, {fopt, xi, direc, iter, funcalls, warnflag}, {allvecs})

      xopt : ndarray
        minimizer of function

      fopt : number
        value of function at minimum: fopt = func(xopt)
      direc -- current direction set
      iter : number
        number of iterations
      funcalls : number 
        number of function calls
      warnflag : number
        Integer warning flag:
                  1 : 'Maximum number of function evaluations.'
                  2 : 'Maximum number of iterations.'
      allvecs : Python list
        a list of solutions at each iteration

    :OtherParameters:

      xtol : number
        line-search error tolerance.
      ftol : number
        acceptable relative error in func(xopt) for convergence.
      maxiter : number
        the maximum number of iterations to perform.
      maxfun : number
        the maximum number of function evaluations.
      full_output : number
        non-zero if fval and warnflag outputs are desired.
      disp : number
        non-zero to print convergence messages.
      retall : number
        non-zero to return a list of the solution at each iteration

    :SeeAlso:

      fmin, fmin_powell, fmin_cg,
             fmin_bfgs, fmin_ncg -- multivariate local optimizers
      leastsq -- nonlinear least squares minimizer

      fmin_l_bfgs_b, fmin_tnc,
             fmin_cobyla -- constrained multivariate optimizers

      anneal, brute -- global optimizers

      fminbound, brent, golden, bracket -- local scalar minimizers

      fsolve -- n-dimenstional root-finding

      brentq, brenth, ridder, bisect, newton -- one-dimensional root-finding

      fixed_point -- scalar fixed-point finder
      
    Notes
    
    -----------------------

      Uses a modification of Powell's method to find the minimum of a function
      of N variables
      """
    # we need to use a mutable object here that we can update in the
    # wrapper function
    fcalls, func = wrap_function(func, args)
    x = asarray(x0).flatten()
    if retall:
        allvecs = [x]
    N = len(x)
    rank = len(x.shape)
    if not -1 < rank < 2:
        raise ValueError, "Initial guess must be a scalar or rank-1 sequence."
    if maxiter is None:
        maxiter = N * 1000
    if maxfun is None:
        maxfun = N * 1000


    if direc is None:
        direc = eye(N, dtype=float)
    else:
        direc = asarray(direc, dtype=float)

    fval = squeeze(func(x))
    x1 = x.copy()
    iter = 0;
    ilist = range(N)
    while True:
        fx = fval
        bigind = 0
        delta = 0.0
        for i in ilist:
            direc1 = direc[i]
            fx2 = fval
            fval, x, direc1 = _linesearch_powell(func, x, direc1, tol=xtol*100)
            if (fx2 - fval) > delta:
                delta = fx2 - fval
                bigind = i
        iter += 1
        if callback is not None:
            callback(x)
        if retall:
            allvecs.append(x)
        if (2.0*(fx - fval) <= ftol*(abs(fx)+abs(fval))+1e-20): break
        if fcalls[0] >= maxfun: break
        if iter >= maxiter: break

        # Construct the extrapolated point
        direc1 = x - x1
        x2 = 2*x - x1
        x1 = x.copy()
        fx2 = squeeze(func(x2))

        if (fx > fx2):
            t = 2.0*(fx+fx2-2.0*fval)
            temp = (fx-fval-delta)
            t *= temp*temp
            temp = fx-fx2
            t -= delta*temp*temp
            if t < 0.0:
                fval, x, direc1 = _linesearch_powell(func, x, direc1, tol=xtol*100)
                direc[bigind] = direc[-1]
                direc[-1] = direc1

    warnflag = 0
    if fcalls[0] >= maxfun:
        warnflag = 1
        if disp:
            print "Warning: Maximum number of function evaluations has "\
                  "been exceeded."
    elif iter >= maxiter:
        warnflag = 2
        if disp:
            print "Warning: Maximum number of iterations has been exceeded"
    else:
        if disp:
            print "Optimization terminated successfully."
            print "         Current function value: %f" % fval
            print "         Iterations: %d" % iter
            print "         Function evaluations: %d" % fcalls[0]

    x = squeeze(x)

    if full_output:
        retlist = x, fval, direc, iter, fcalls[0], warnflag
        if retall:
            retlist += (allvecs,)
    else:
        retlist = x
        if retall:
            retlist = (x, allvecs)

    return retlist




def _endprint(x, flag, fval, maxfun, xtol, disp):
    if flag == 0:
        if disp > 1:
            print "\nOptimization terminated successfully;\n the returned value" + \
                  " satisfies the termination criteria\n (using xtol = ", xtol, ")"
    if flag == 1:
        print "\nMaximum number of function evaluations exceeded --- increase maxfun argument.\n"
    return


def brute(func, ranges, args=(), Ns=20, full_output=0, finish=fmin):
    """Minimize a function over a given range by brute force.

    :Parameters:

    func -- Function to be optimized
    ranges : tuple 
        Tuple where each element is a tuple of parameters
                      or a slice object to be handed to numpy.mgrid

    args  -- Extra arguments to function.
    Ns : number 
        Default number of samples if not given
    full_output : number 
        Nonzero to return evaluation grid.

    :Returns: (x0, fval, {grid, Jout})

    x0 : ndarray
        Value of arguments giving minimum over the grird
    fval : number
        Function value at minimum
    grid : tuple
        tuple with same length as x0 representing the  evaluation grid
    Jout : ndarray -- Function values over grid:  Jout = func(*grid)

    :SeeAlso:

      fmin, fmin_powell, fmin_cg,
             fmin_bfgs, fmin_ncg -- multivariate local optimizers
      leastsq -- nonlinear least squares minimizer

      fmin_l_bfgs_b, fmin_tnc,
             fmin_cobyla -- constrained multivariate optimizers

      anneal, brute -- global optimizers

      fminbound, brent, golden, bracket -- local scalar minimizers

      fsolve -- n-dimenstional root-finding

      brentq, brenth, ridder, bisect, newton -- one-dimensional root-finding

      fixed_point -- scalar fixed-point finder

    Notes
    
    ------------------
    
    Find the minimum of a function evaluated on a grid given by the tuple ranges.
    """
    N = len(ranges)
    if N > 40:
        raise ValueError, "Brute Force not possible with more than 40 variables."
    lrange = list(ranges)
    for k in range(N):
        if type(lrange[k]) is not type(slice(None)):
            if len(lrange[k]) < 3:
                lrange[k] = tuple(lrange[k]) + (complex(Ns),)
            lrange[k] = slice(*lrange[k])
    if (N==1):
        lrange = lrange[0]

    def _scalarfunc(*params):
        params = squeeze(asarray(params))
        return func(params,*args)

    vecfunc = vectorize(_scalarfunc)
    grid = mgrid[lrange]
    if (N==1):
        grid = (grid,)
    Jout = vecfunc(*grid)
    Nshape = shape(Jout)
    indx = argmin(Jout.ravel(),axis=-1)
    Nindx = zeros(N,int)
    xmin = zeros(N,float)
    for k in range(N-1,-1,-1):
        thisN = Nshape[k]
        Nindx[k] = indx % Nshape[k]
        indx = indx / thisN
    for k in range(N):
        xmin[k] = grid[k][tuple(Nindx)]

    Jmin = Jout[tuple(Nindx)]
    if (N==1):
        grid = grid[0]
        xmin = xmin[0]
    if callable(finish):
        vals = finish(func,xmin,args=args,full_output=1, disp=0)
        xmin = vals[0]
        Jmin = vals[1]
        if vals[-1] > 0:
            print "Warning: Final optimization did not succeed"
    if full_output:
        return xmin, Jmin, grid, Jout
    else:
        return xmin


def main():
    import time

    times = []
    algor = []
    x0 = [0.8,1.2,0.7]
    print "Nelder-Mead Simplex"
    print "==================="
    start = time.time()
    x = fmin(rosen,x0)
    print x
    times.append(time.time() - start)
    algor.append('Nelder-Mead Simplex\t')

    print
    print "Powell Direction Set Method"
    print "==========================="
    start = time.time()
    x = fmin_powell(rosen,x0)
    print x
    times.append(time.time() - start)
    algor.append('Powell Direction Set Method.')

    print
    print "Nonlinear CG"
    print "============"
    start = time.time()
    x = fmin_cg(rosen, x0, fprime=rosen_der, maxiter=200)
    print x
    times.append(time.time() - start)
    algor.append('Nonlinear CG     \t')

    print
    print "BFGS Quasi-Newton"
    print "================="
    start = time.time()
    x = fmin_bfgs(rosen, x0, fprime=rosen_der, maxiter=80)
    print x
    times.append(time.time() - start)
    algor.append('BFGS Quasi-Newton\t')

    print
    print "BFGS approximate gradient"
    print "========================="
    start = time.time()
    x = fmin_bfgs(rosen, x0, gtol=1e-4, maxiter=100)
    print x
    times.append(time.time() - start)
    algor.append('BFGS without gradient\t')


    print
    print "Newton-CG with Hessian product"
    print "=============================="
    start = time.time()
    x = fmin_ncg(rosen, x0, rosen_der, fhess_p=rosen_hess_prod, maxiter=80)
    print x
    times.append(time.time() - start)
    algor.append('Newton-CG with hessian product')


    print
    print "Newton-CG with full Hessian"
    print "==========================="
    start = time.time()
    x = fmin_ncg(rosen, x0, rosen_der, fhess=rosen_hess, maxiter=80)
    print x
    times.append(time.time() - start)
    algor.append('Newton-CG with full hessian')

    print
    print "\nMinimizing the Rosenbrock function of order 3\n"
    print " Algorithm \t\t\t       Seconds"
    print "===========\t\t\t      ========="
    for k in range(len(algor)):
        print algor[k], "\t -- ", times[k]

if __name__ == "__main__":
    main()



#! /usr/bin/env python

import aipy, ephem, numpy

ra,dec,epoch = ('19:59:28.3', '+40:44:02', ephem.J2000)
cyg = ephem.Equatorial(ra, dec, epoch=epoch)
print cyg.get()
ra,dec = cyg.get()
v_eq = aipy.coord.radec2eq((ra,dec))
print v_eq, 'J2000'
m = aipy.coord.convert_m('eq','eq', iepoch=epoch, oepoch='2007.5')
pv_eq = numpy.dot(m, v_eq)
print pv_eq, 'J2007.5'
ra,dec = aipy.coord.eq2radec(pv_eq)
pcyg = ephem.Equatorial(ra, dec, epoch='2007.5')
print pcyg.get()
pcyg = ephem.Equatorial(cyg, epoch='2007.5')
print 'Correct answer is: (19:59:43.88, 40:45:16.9)'



"""
Provides interfaces to Healpix_cxx, which was developed at the 
Max-Planck-Institut fuer Astrophysik and financially supported by the 
Deutsches Zentrum fuer Luft- und Raumfahrt (DLR).
Adds data to the HealpixBase class using numpy arrays, and interfaces to
FITS files using pyfits.
"""

import numpy as n, utils, pyfits
from _healpix import HealpixBase
from _alm import Alm

default_fits_format_codes = {
    n.bool_:'L', n.uint8:'B', n.int16:'I', n.int32:'J', n.int64:'K', 
    n.float32:'E', n.float64:'D', n.complex64:'C', n.complex128:'M'
}

def mk_arr(val, dtype=n.double):
    if type(val) is n.ndarray: return val.astype(dtype)
    return n.array(val, dtype=dtype).flatten()

class HealpixMap(HealpixBase):
    """Collection of utilities for mapping data on a sphere.  Adds a data map 
    to the infrastructure in _healpix.HealpixBase."""
    def __init__(self, *args, **kwargs):
        dtype = kwargs.pop('dtype', n.double)
        interp = kwargs.pop('interp', False)
        fromfits = kwargs.pop('fromfits', None)
        HealpixBase.__init__(self, *args, **kwargs)
        self._use_interpol = interp
        if fromfits is None:
            m = n.zeros((self.npix(),), dtype=dtype)
            self.set_map(m, scheme=self.scheme())
        else: self.from_fits(fromfits)
    def set_interpol(self, onoff):
        """Choose whether __getitem___ (i.e. HealpixMap[crd]) returns an
        interpolated value or just nearest pixel.  Default upon creating a
        HealpixMap is to not interpolate."""
        self._use_interpol = onoff
    def set_map(self, data, scheme="RING"):
        """Assign data to HealpixMap.map.  Infers Nside from # of pixels via
        Npix = 12 * Nside**2."""
        try:
            assert(data.ndim == 1)
            nside = self.npix2nside(data.shape[0])
        except(AssertionError,ValueError):
            raise ValueError("Data must be a 1 dim array with 12*N**2 entries.")
        self.set_nside_scheme(nside, scheme)
        self.map = data
    def get_map(self):
        """Return Healpix data as a 1 dimensional numpy array."""
        return self.map
    def change_scheme(self, scheme):
        """Reorder the pixels in map to be "RING" or "NEST" ordering."""
        assert(scheme in ["RING", "NEST"])
        if scheme == self.scheme(): return
        i = self.nest_ring_conv(n.arange(self.npix()), scheme)
        self[i] = self.map
        self.set_nside_scheme(self.nside(), scheme)
    def __getitem__(self, crd):
        """Access data on a sphere via hpm[crd].
        crd = either 1d array of pixel indices, (th,phi), or (x,y,z), where
        th,phi,x,y,z are numpy arrays of coordinates."""
        if type(crd) is tuple:
            crd = [mk_arr(c, dtype=n.double) for c in crd]
            if self._use_interpol:
                px,wgts = self.crd2px(*crd, **{'interpolate':1})
                return n.sum(self.map[px] * wgts, axis=-1)
            else: px = self.crd2px(*crd)
        else: px = mk_arr(crd, dtype=n.long)
        return self.map[px]
    def __setitem__(self, crd, val):
        """Assign data to a sphere via hpm[crd] = val.  Functionality slightly
        complicated to make repeat coordinates assign sum of values (i.e.
        crd = ([1,1], [2,2]), val = [3,3] will assign 6 to location (1,2).
        crd = either 1d array of pixel indices, (th,phi), or (x,y,z), where
        th,phi,x,y,z are numpy arrays of coordinates."""
        if type(crd) is tuple:
            crd = [mk_arr(c, dtype=n.double) for c in crd]
            px = self.crd2px(*crd)
        else:
            if type(crd) is n.ndarray: assert(len(crd.shape) == 1)
            px = mk_arr(crd, dtype=n.int)
        if px.size == 1: self.map[px] = val
        else:
            m = n.zeros_like(self.map)
            px = px.reshape(px.size,1)
            cnt = n.zeros(self.map.shape, dtype=n.bool)
            val = mk_arr(val, dtype=m.dtype)
            utils.add2array(m, px, val)
            utils.add2array(cnt, px, n.ones(val.shape, dtype=n.bool))
            self.map = n.where(cnt, m, self.map)
    def from_hpm(self, hpm):
        """Initialize this HealpixMap with data from another.  Takes care
        of upgrading or downgrading the resolution, and swaps ordering
        scheme if necessary."""
        if hpm.nside() < self.nside():
            interpol = hpm._use_interpol
            hpm.set_interpol(True)
            px = n.arange(self.npix())
            th,phi = self.px2crd(px, ncrd=2)
            self[px] = hpm[th,phi]
        elif hpm.nside() > self.nside():
            factor = hpm.nside() / self.nside()
            px = n.arange(hpm.npix())
            th,phi = hpm.px2crd(px, ncrd=2)
            self[th,phi] = hpm[px] / factor
        else:
            if hpm.scheme() == self.scheme(): self.map = hpm.map
            else:
                i = self.nest_ring_conv(n.arange(self.npix()), hpm.scheme())
                self.map = hpm.map[i]
    def from_alm(self, alm):
        """Set data to the map generated by the spherical harmonic
        coefficients contained in alm."""
        self.set_map(alm.to_map(self.nside(), self.scheme()))
    def to_alm(self, lmax, mmax, iter=1):
        """Return an Alm object containing the spherical harmonic components
        of a map (in RING mode) up to the specified lmax,mmax.  Greater
        accuracy can be achieved by increasing iter."""
        assert(self.scheme() == 'RING')
        alm = Alm(lmax,mmax)
        alm.from_map(self.map, iter)
        return alm
    def from_fits(self, filename, hdunum=1, colnum=0):
        """Read a HealpixMap from the specified location in a fits file."""
        hdu = pyfits.open(filename)[hdunum]
        data = hdu.data.field(colnum)
        scheme= hdu.header['ORDERING'][:4]
        self.set_map(data, scheme=scheme)
    def _set_fits_header(self, hdr):
        hdr.update('PIXTYPE', 'HEALPIX', 'HEALPIX pixelisation')
        scheme = self.scheme()
        if scheme == 'NEST': scheme == 'NESTED'
        hdr.update('ORDERING', scheme,
            'Pixel ordering scheme, either RING or NESTED')
        hdr.update('NSIDE', self.nside(), 'Resolution parameter for HEALPIX')
        hdr.update('FIRSTPIX', 0, "First pixel # (0 based)")
        hdr.update('LASTPIX', self.npix()-1, "Last pixel # (0 based)")
        hdr.update('INDXSCHM', 'IMPLICIT', "Indexing: IMPLICIT or EXPLICIT")
    def to_fits(self, filename, format=None):
        """Write a HealpixMap to a fits file in the fits format specified by
        'format'.  Default uses mapping of numpy types to fits format types
        stored in default_fits_format_codes."""
        if format is None:
            format = default_fits_format_codes[self.map.dtype.type]
        hdu0 = pyfits.PrimaryHDU()
        col0 = pyfits.Column(name='signal', format=format, array=self.map)
        cols = pyfits.ColDefs([col0])
        tbhdu = pyfits.new_table(cols)
        self._set_fits_header(tbhdu.header)
        hdulist = pyfits.HDUList([hdu0, tbhdu])
        hdulist.writeto(filename)




#! /usr/bin/env python
import numpy as n, aipy as a, pyfits

im = a.img.Img(size=200, res=.5)
L,M = im.get_LM(center=(200,200))
d_ra = n.arcsin(L[199,199]) * a.img.rad2deg
d_dec = n.arcsin(M[199,199]) * a.img.rad2deg

a.img.to_fits('test_img.fits', L.filled(0), clobber=True, 
    ra=10, d_ra=d_ra, dec=40, d_dec=d_dec)
d,h = a.img.from_fits('test_img.fits')
d = d.squeeze()
print d.shape
print d
print h
